# -*- coding: latin-1 -*-

import copy, time
import unittest

from nive.tests.test_mysql import ENABLE_MYSQL_TESTS, myapp

class utc:
    pass
uTestCase = utc

if ENABLE_MYSQL_TESTS:
    from nive.utils.dataPool2.mySqlPool import *
    uTestCase = unittest.TestCase
    
from nive.definitions import DatabaseConf
from nive.utils.dataPool2.base import *
from nive.utils.path import DvPath

from test_db import dbTest
from test_Base import conf, stdMeta, struct, SystemFlds, Fulltext, Files, data1_1, data2_1, meta1, file1_1, file1_2

from nive.tests import __local

conn = DatabaseConf(
    user = __local.USER,
    password = __local.PASSWORD,
    host = __local.HOST,
    port = __local.PORT,
    dbName = __local.DATABASE,
    unicode = 1,
    timeout = 1
)
myconn = conn

def getPool():
    p = MySql(connParam=myconn, **conf)
    p.structure.Init(structure=struct, stdMeta=struct[u"pool_meta"])
    return p


class MySqlTest(dbTest, uTestCase):
    """
    """

    def setUp(self):
        self.pool = getPool()
        self.checkdb()
        self.connect()

    def connect(self):
        #print "Connect DB on", conn["host"],
        self.pool.CreateConnection(myconn)
        self.assert_(self.pool.connection.IsConnected())
        #print "OK"

    def checkdb(self):
        myapp()
    
def __test():
    unittest.main()



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------
__doc__ = """
This file contains the main application functionality. The application handles configuration
registration, roots and the database. Also it provides convenient functions to lookup all
different kinds of configurations. 

Application.modules stores configuration objects registered by calling `Register()`. 

See Application.Startup() for the main entry point and connected events. 

Not to be mixed up with pyramid applications or projects. It is possible to use multiple nive
applications in a single pyramid app.
"""
from nive import __version__

import copy
import logging

from time import time
from types import DictType

from zope.interface.registry import Components
from zope.interface import providedBy

from nive.utils.dataPool2.structure import PoolStructure
from nive.i18n import _

from nive.definitions import AppConf, DatabaseConf, SystemFlds, MetaTbl, ReadonlySystemFlds
from nive.definitions import IViewModuleConf, IViewConf, IRootConf, IObjectConf, IToolConf 
from nive.definitions import IAppConf, IDatabaseConf, IModuleConf, IWidgetConf

from nive.security import User, authenticated_userid
from nive.helper import ResolveName, ResolveConfiguration, FormatConfTestFailure, GetClassRef, ClassFactory
from nive.tools import _IGlobal, _GlobalObject
from nive.workflow import IWfProcessConf
from nive.utils.utils import SortConfigurationList


class Application(object):
    """
    nive Application implementaion.

    SQLite example:
    ::
    
        conf = AppConf(id="website", title="nive app", 
                       dbConfiguration = DatabaseConf(context="Sqlite3", 
                                                      fileRoot="data", 
                                                      dbName="data/ewp.db")
        )
    
    MySql example:
    ::
    
        conf = AppConf(id="website", title="nive app", 
                       dbConfiguration = DatabaseConf(context="MySql", 
                                                      fileRoot="data", 
                                                      dbName="nive",
                                                      host="localhost",
                                                      user="user",
                                                      password="password")
        )

    Requires (Configuration, Registration, AppFactory, Events.Events)
    """
    
    def __init__(self, configuration=None):
        """
        Use configuration and dbConfiguration objects to setup your application.
        
        Events:
        - init(configuration)
        """
        self.registry = Components()
        self.configuration = AppConf()
        self.dbConfiguration = DatabaseConf()
        # set id for logging
        if configuration:
            self.id = configuration.id
        else:
            self.id = __name__
        
        self.__name__ = u""
        self.__parent__ = None
        self.__acl__ = []

        # development
        self.debug = False
        self.reloadExtensions = False
    
        # internal configuration
        self._defaultRoot = ""
        self._meta = copy.deepcopy(SystemFlds)
        self._views = []
        # cache database structure 
        self._structure = PoolStructure()
        self._dbpool = None
        log = logging.getLogger(self.id)
        
        log.debug("Initialize %s", repr(configuration))
        self.Signal("init", configuration=configuration)

        if configuration:
            self.Register(configuration)
        
    def __del__(self):
        self.Close()
        

    def Startup(self, pyramidConfig, debug=False):
        """
        Called by nive.portal.portal on application startup.

        Events (in order):

        - startup(app)
        - startRegistration(app, pyramidConfig)
        - finishRegistration(app, pyramidConfig)
        - run(app)
        """
        t = time()
        log = logging.getLogger(self.id)
        log.debug("Startup with debug=%s", str(debug))
        self.Signal("startup", app=self)
        self.SetupRegistry()
        self.debug = debug
        # disable caching during startup
        #cache = self.configuration.useCache
        #self.configuration.useCache = False
        
        self.Signal("startRegistration", app=self, pyramidConfig=pyramidConfig)
        # register pyramid views 
        if pyramidConfig:
            pyramidConfig = self.RegisterViewModules(pyramidConfig)
            pyramidConfig.commit()
            pyramidConfig = self.RegisterViews(pyramidConfig)
            pyramidConfig.commit()

        # register groups
        portal = self.portal
        if portal and hasattr(portal, "RegisterGroups"):
            portal.RegisterGroups(self)
        self.Signal("finishRegistration", app=self, pyramidConfig=pyramidConfig)
        log.debug('Finished registration.')
        
        # reload database structure
        self.LoadStructure(forceReload = True)
        self._dbpool = self._GetDataPoolObj()
                           
        # test and create database fields 
        if debug:
            self.GetTool("nive.components.tools.dbStructureUpdater", self).Run()
            result, report = self.TestDB()
            if not result:
                log.error('Database test result: %s %s', str(result), ", ".join(report))
            else:
                log.info('Database test result: %s %s', str(result), ", ".join(report))
            if not self._structure.get(u"pool_meta"):
                log.error('No meta fields in _structure %s', repr(self._structure))
 
        # reset caching after startup
        #self.configuration.useCache = cache
        self._Lock()
        
        # start
        self.Signal("run", app=self)
        log.info('Application running. Runtime logging as [%s]. Startup time: %.05f.', self.configuration.id, time()-t)
        self.id = self.configuration.id

    
    def Close(self):
        """
        Close database and roots.
        """
        self._CloseRootObj()
        if self._dbpool:
            self._dbpool.Close()



    # Properties -----------------------------------------------------------

    def root(self, name=""):
        """ 
        Events:
        - loadRoot(root)

        returns root object
        """
        return self._GetRootObj(name)

    def obj(self, id, rootname = "", **kw):
        """ returns object    """
        return self.LookupObj(id, rootname, **kw)

    @property
    def db(self):
        """ returns datapool object    """
        return self._dbpool

    @property
    def portal(self):
        """ returns the portal    """
        return self.__parent__

    @property
    def app(self):
        """ returns itself. for compatibility.    """
        return self

    # Content and root objects -----------------------------------------------------------

    def __getitem__(self, name):
        """
        This function is called by url traversal and looks up root objects for the corresponding
        name. 
        """
        name = name.split(".")[0]
        o = self.root(name)
        if o and o.configuration.urlTraversal:
            return o
        raise KeyError, name


    def GetRoot(self, name=""):
        """
        Returns the data root object. If name is empty the default root is returned.
        
        Events:
        - loadRoot(root)

        returns root object
        """
        return self._GetRootObj(name)


    def GetRoots(self):
        """
        Returns all root objects.
        
        returns list
        """
        return [self.GetRoot(r.id) for r in self.GetAllRootConfs()]


    def GetApp(self):
        return self


    def GetTool(self, toolID, contextObject=None):
        """
        Load tool object. *toolID* must be the tool.id or dotted python name.
        
        returns the tool object.
        """
        return self._GetToolObj(toolID, contextObject or _GlobalObject())


    def GetWorkflow(self, wfProcID, contextObject=None):
        """
        Load workflow process. *wfProcID* must be the wf.id or dotted python name.
        
        returns the workflow object
        """
        if not self.configuration.workflowEnabled or wfProcID == "":
            return None
        return self._GetWfObj(wfProcID, contextObject or _GlobalObject())


    # Data Pool ------------------------------------------------------------------------------

    def GetDB(self):
        """
        Create data pool object and database connection.
        
        returns the datapool object
        """
        return self._dbpool


    def LookupObj(self, id, rootname = "", **kw):
        """
        Returns the object for the id. if root name is empty the default root is used.
        
        returns object
        """
        root = self.GetRoot(rootname)
        return root.LookupObj(id, **kw)


    def Query(self, sql, values = []):
        """
        Start a sql query on the database. 
        
        returns tuple or none
        """
        db = self.db
        r = db.Query(sql, values)
        return r


    def TestDB(self):
        """
        Test database connection for errors. Returns state and list with errors.
        
        returns bool, list
        """
        try:
            e = []
            db = self.db
            if not db.connection.IsConnected():
                return False, e
            return True, e

        except Exception, err:
            e.append(str(err))
            return False, e


    def NewConnection(self):
        """
        Creates a new database connection. Works before startup and application setup.
        """
        return self._GetConnection()
        

    def NewDBApi(self):
        """
        Creates a raw database connection (dbapi). Works before startup and application setup.
        """
        return self._GetDBApi()
        

    def GetVersion(self):
        """ """
        return __version__, __version__

    def CheckVersion(self):
        """ """
        return __version__ == __version__


    # to be removed in future versions -------------------------------------------

    def NewDBConnection(self):
        return self.NewConnection()
    
    def ConvertID(self, id):
        try:
            return int(id)
        except:
            return -1

    def GetCountEntries(self):
        db = self.db
        c = db.GetCountEntries()
        return c




class Registration(object):

    def Register(self, module, **kw):
        """
        Include a module or configuration to store in the registry.

        Handles configuration objects with the following interfaces:
        
        - IAppConf
        - IRootConf
        - IObjectConf
        - IViewModuleConf
        - IViewConf 
        - IToolConf
        - IModuleConf
        - IWidgetConf
        - IWfProcessConf
        
        Other modules are registered as utility with **kws as parameters.
        
        raises TypeError, ConfigurationError, ImportError
        """
        log = logging.getLogger(self.id)
        iface, conf = ResolveConfiguration(module)
        if not conf:
            try:
                log.debug('Register module: %s', str(module))
                return self.registry.registerUtility(module, **kw)
            except Exception, e:
                raise ConfigurationError, str(module)
        
        # test conf
        if self.debug:
            r=conf.test()
            if r:
                v = FormatConfTestFailure(r)
                log.warn('Configuration test failed:\r\n%s', v)
                #return False
        
        log.debug('Register module: %s %s', str(conf), str(iface))
        # register module views
        if iface not in (IViewModuleConf, IViewConf):
            self.RegisterConfViews(conf)
            
        # reset cached class value. makes testing easier
        try:
            del conf._v_class
        except:
            pass
            
        # register module itself
        if hasattr(conf, "unlock"):
            conf.unlock()
        
        # events
        if conf.get("events"):
            for e in conf.events:
                log.debug('Register Event: %s for %s', str(e.event), str(e.callback))
                self.ListenEvent(e.event, e.callback)
        
        if iface == IRootConf:
            self.registry.registerUtility(conf, provided=IRootConf, name=conf.id)
            if conf.default or not self._defaultRoot:
                self._defaultRoot = conf.id
            return True
        elif iface == IObjectConf:
            self.registry.registerUtility(conf, provided=IObjectConf, name=conf.id)
            return True

        elif iface == IViewConf:
            self.registry.registerUtility(conf, provided=IViewConf, name=conf.id)
            return True
        elif iface == IViewModuleConf:
            self.registry.registerUtility(conf, provided=IViewModuleConf, name=conf.id)
            if conf.widgets:
                for w in conf.widgets:
                    self.Register(w)
            return True

        elif iface == IToolConf:
            if conf.apply:
                for i in conf.apply:
                    if i == None:
                        self.registry.registerAdapter(conf, (_IGlobal,), IToolConf, name=conf.id)
                    else:
                        self.registry.registerAdapter(conf, (i,), IToolConf, name=conf.id)
            else:
                self.registry.registerAdapter(conf, (_IGlobal,), IToolConf, name=conf.id)
            return True

        elif iface == IAppConf:
            self.registry.registerUtility(conf, provided=IAppConf, name="IApp")
            if conf.modules:
                for m in conf.modules:
                    self.Register(m)
            return True
        
        elif iface == IDatabaseConf:
            self.registry.registerUtility(conf, provided=IDatabaseConf, name="IDatabase")
            return True
        
        elif iface == IModuleConf:
            # modules
            if conf.modules:
                for m in conf.modules:
                    self.Register(m, **kw)
            self.registry.registerUtility(conf, provided=IModuleConf, name=conf.id)
            return True

        elif iface == IWidgetConf:
            for i in conf.apply:
                self.registry.registerAdapter(conf, (i,), conf.widgetType, name=conf.id)
            return True

        elif iface == IWfProcessConf:
            if conf.apply:
                for i in conf.apply:
                    self.registry.registerAdapter(conf, (i,), IWfProcessConf, name=conf.id)
            else:
                self.registry.registerAdapter(conf, (_IGlobal,), IWfProcessConf, name=conf.id)
            return True

        raise TypeError, "Unknown configuration interface type (%s)" % (str(conf))
        
        
        
    def RegisterConfViews(self, conf):    
        # register views included in configuration.views
        views = None
        try: 
            views = conf.views
        except:
            pass
        if not views:
            return
        for v in views:

            if isinstance(v, basestring):
                iface, conf = ResolveConfiguration(v)
                if not conf:
                    raise ConfigurationError, str(v)
                v = conf
                
            self.Register(v)


    def SetupRegistry(self):
        """
        Loads self.configuration, includes modules and updates meta fields.
        """
        c = self.registry.queryUtility(IAppConf, name="IApp")
        if c:
            self.configuration = c
        # bw 0.9.3
        if not self.configuration:
            raise TypeError, "Application configuration is not set"
        
        def idinlist(l, id):
            for i in l:
                if i["id"] == id:
                    return True
            return False

        c = self.configuration
        for k in c.keys():
            # special values
            if k == "id" and c.id:
                self.__name__ = c.id
            if k == "acl" and c.acl:
                self.__acl__ = c.acl
                continue
            if k == "meta" and c.meta:
                temp = []
                for system in self._meta:
                    if idinlist(c.meta, system["id"]):
                        continue
                    temp.append(system)
                for m in c.meta:
                    temp.append(m)
                self._meta = temp
                continue
            if k == "dbConfiguration" and c.dbConfiguration:
                if type(c.dbConfiguration) == DictType:
                    self.dbConfiguration = DatabaseConf(**c.dbConfiguration)
                else:
                    self.dbConfiguration = c.dbConfiguration
                continue
            
            # map value
            #setattr(self, k, c[k])

        dbc = self.registry.queryUtility(IDatabaseConf, name="IDatabase")
        # bw 0.9.3
        if dbc:
            self.dbConfiguration = dbc
        

    def RegisterViews(self, config):
        """
        Register configured views and static views with the pyramid web framework.
        """
        views = self.registry.getAllUtilitiesRegisteredFor(IViewConf)
        # single views
        for view in views:
            config.add_view(view=view.view,
                            context=view.context,
                            attr=view.attr,
                            name=view.name,
                            renderer=view.renderer,
                            permission=view.permission,
                            containment=view.containment,
                            custom_predicates=view.custom_predicates or (self.AppViewPredicate,),
                            **view.options)
            #try:
            #except Exception, e:
            #    pass
        return config


    def RegisterViewModules(self, config):
        """
        Register configured views and static views with the pyramid web framework.
        """
        mods = self.registry.getAllUtilitiesRegisteredFor(IViewModuleConf)
        for viewmod in mods:
            # object views
            for view in viewmod.views:
                config.add_view(attr=view.attr,
                                name=view.name,
                                view=view.view or viewmod.view,
                                context=view.context or viewmod.context,
                                renderer=view.renderer or viewmod.renderer,
                                permission=view.permission or viewmod.permission,
                                containment=view.containment or viewmod.containment,
                                custom_predicates=view.custom_predicates or viewmod.custom_predicates or (self.AppViewPredicate,),
                                **view.options)
                #try:
                #except Exception, e:
                #    raise ConfigurationError, str(view)
            
            # static views
            maxage = 60*60*4
            if self.debug:
                maxage = None
            config.add_static_view(name=viewmod.staticName or viewmod.id, path=viewmod.static, cache_max_age=maxage)
        
        return config


    def AppViewPredicate(self, context, request):
        """
        Check if context of view is this application. For multisite support.
        """
        app = context.app
        return app == self
    AppViewPredicate.__text__ = "nive.AppViewPredicate"
        
        
    def _Lock(self):
        # lock all configurations
        # adapters
        for a in self.registry.registeredAdapters():
            try:       a.factory.lock()
            except:    pass
        for a in self.registry.registeredUtilities():
            try:       a.component.lock()
            except:    pass


    # bw 0.9.3
    def Include(self, module, **kw):
        return self.Register(module, **kw)

    def LoadConfiguration(self):
        return self.SetupRegistry()

    
    


class Configuration:
    """
    Read access functions for root, type, type field, meta field and category configurations.

    Requires:
    - nive.nive
    """

    def QueryConf(self, queryFor, context=None):
        """
        returns configuration or None
        """
        if isinstance(queryFor, basestring):
            queryFor = ResolveName(queryFor)
        if context:
            return self.registry.getAdapters((context,), queryFor)
        return self.registry.getAllUtilitiesRegisteredFor(queryFor)

    def QueryConfByName(self, queryFor, name, context=None):
        """
        returns configuration or None
        """
        if isinstance(queryFor, basestring):
            queryFor = ResolveName(queryFor)
        if context:
            return self.registry.queryAdapter(context, queryFor, name=name)
        return self.registry.queryUtility(queryFor, name=name)
    
    
    def Factory(self, queryFor, name, context=None):
        """
        Query for configuration and lookup class reference. Does not call __init__ for
        the new class.
        
        returns class or None
        """
        c = self.QueryConfByName(queryFor, name, context=context)
        if not c:
            return None
        return ClassFactory(c)


    # Roots -------------------------------------------------------------

    def GetRootConf(self, name=""):
        """
        Get the root object configuration. If name is empty, the default name is used.
        
        returns dict or None
        """
        if name == "":
            name = self._defaultRoot
        return self.registry.queryUtility(IRootConf, name=name)


    def GetAllRootConfs(self):
        """
        Get all root object configurations as list.
        
        returns list
        """
        return self.registry.getAllUtilitiesRegisteredFor(IRootConf)


    def GetRootIds(self):
        """
        Get all root object ids.
        
        returns list
        """
        return [r["id"] for r in self.GetAllRootConfs()]


    def GetDefaultRootName(self):
        """
        Returns the name of the default root.
        
        returns string
        """
        return self._defaultRoot


    # Types -------------------------------------------------------------

    def GetObjectConf(self, typeID, skipRoot=False):
        """
        Get the type configuration for typeID. If type is not found root definitions are
        searched as well.
        
        returns configuration or none
        """
        c = self.registry.queryUtility(IObjectConf, name=typeID)
        if c or skipRoot:
            return c
        return self.GetRootConf(typeID)


    def GetAllObjectConfs(self, visibleOnly = False):
        """
        Get all type configurations.
        
        returns list
        """
        c = self.registry.getAllUtilitiesRegisteredFor(IObjectConf)
        if not visibleOnly:
            return c
        return filter(lambda t: t.get("hidden") != True, c)


    def GetTypeName(self, typeID):
        """
        Get the object type name for the id.
        
        returns string
        """
        t = self.GetObjectConf(typeID)
        if t:
            return t.name
        return ""


    # Tool -------------------------------------------------------------

    def GetToolConf(self, toolID, contextObject=None):
        """
        Get the tool configuration.
        
        returns configuration or None
        """
        if not contextObject:
            contextObject = _GlobalObject()
        v = self.registry.queryAdapter(contextObject, IToolConf, name=toolID)
        if v:
            return v[0]
        return None

    def GetAllToolConfs(self, contextObject=None):
        """
        Get all tool configurations.
        
        returns list
        """
        tools = []
        for a in self.registry.registeredAdapters():
            if a.provided==IToolConf:
                if contextObject:
                    if not a.required[0] in providedBy(contextObject):
                        continue
                tools.append(a.factory)
        return tools


    # General Fields -------------------------------------------------------------

    def GetFld(self, fldID, typeID = None):
        """
        Get type or meta field definition. Type fields have the priority if typeID is not None.

        returns configuration or None
        """
        if typeID:
            f = self.GetObjectFld(fldID, typeID)
            if f:
                return f
        return self.GetMetaFld(fldID)


    def GetFldName(self, fldID, typeID = None):
        """
        Get type or meta field name. Type fields have the priority if typeID is not None.
        
        returns string
        """
        if typeID:
            f = self.GetObjectFld(fldID, typeID)
            if f:
                return f["name"]
        return self.GetMetaFldName(fldID)


    # Type Data Fields -------------------------------------------------------------

    def GetObjectFld(self, fldID, typeID):
        """
        Returns object field configuration.
        
        returns configuration or None
        """
        fields = self.GetAllObjectFlds(typeID)
        if not fields:
            return None
        o = filter(lambda d: d["id"]==fldID, fields)
        if not o:
            return None
        return o[0]


    def GetAllObjectFlds(self, typeID):
        """
        Get all object field configurations.
        
        returns list or None
        """
        t = self.GetObjectConf(typeID)
        if not t:
            return None
        return t.data


    # Meta Fields -------------------------------------------------------------

    def GetMetaFld(self, fldID):
        """
        Get meta field configuration
        
        returns configuration or None
        """
        m = filter(lambda d: d["id"]==fldID, self._meta)
        if not m:
            return None
        return m[0]


    def GetAllMetaFlds(self, ignoreSystem = True):
        """
        Get all meta field configurations.
        
        returns list
        """
        if not ignoreSystem:
            return self._meta
        return filter(lambda m: m["id"] in ReadonlySystemFlds, self._meta)


    def GetMetaFldName(self, fldID):
        """
        Get meta field name for id.
        
        returns string
        """
        m = filter(lambda d: d["id"]==fldID, self._meta)
        if not m:
            return u""
        return m[0]["name"]


    # Categories -------------------------------------------------------------------------------------------------------

    def GetCategory(self, categoryID=""):
        """
        Get category configuration.
        
        returns configuration or None
        """
        c = filter(lambda d: d["id"]==categoryID, self.configuration.categories)
        if not c:
            return None
        return c[0]


    def GetAllCategories(self, sort=u"name", visibleOnly=False):
        """
        Get all category configurations.
        
        returns list
        """
        if not visibleOnly:
            return SortConfigurationList(self.configuration.categories, sort)
        c = filter(lambda a: not a.get("hidden"), self.configuration.categories)
        return SortConfigurationList(c, sort)


    def GetCategoryName(self, categoryID):
        """
        Get category name for id.
        
        returns string
        """
        c = filter(lambda d: d["id"]==categoryID, self.configuration.categories)
        if not c:
            return u""
        return c[0]["name"]


    # Groups -------------------------------------------------------------

    def GetGroups(self, sort=u"name", visibleOnly=False):
        """
        Get a list of configured groups.
        
        returns list
        """
        if not self.configuration.groups:
            return []
        if not visibleOnly:
            return SortConfigurationList(self.configuration.groups, sort)
        c = filter(lambda a: not a.get("hidden"), self.configuration.groups)
        return SortConfigurationList(c, sort)


    def GetGroupName(self, groupID):
        """
        Get group name for id.
        
        returns string
        """
        g = filter(lambda d: d["id"]==groupID, self.configuration.groups)
        if not g:
            return u""
        return g[0]["name"]


    # Workflow -------------------------------------------------------------

    def GetWorkflowConf(self, processID, contextObject=None):
        """
        Get the workflow process configuration.
        
        returns configuration or None
        """
        return self.registry.queryAdapter(contextObject or _GlobalObject(), IWfProcessConf, name=processID)


    def GetAllWorkflowConfs(self, contextObject=None):
        """
        Get all workflow configurations.
        
        returns list
        """
        w = []
        adpts = self.registry.registeredAdapters()
        for a in adpts:
            if a.provided==IWfProcessConf:
                if contextObject:
                    if not a.required[0] in providedBy(contextObject):
                        continue
                w.append(a.factory)
        return w


    # nive Configuration -------------------------------------------------------------

    def StoreSysValue(self, key, value):
        """
        Stores a value in `pool_sys` table. Value must be a string of any size.
        """
        db = self.db
        db.UpdateFields(u"pool_sys", key, {u"id": key, u"value":value, u"ts":time()}, autoinsert=True)
        db.Commit()

    def LoadSysValue(self, key):
        """
        Loads the value stored as `key` from `pool_sys` table. 
        """
        db = self.db
        sql, values = db.FmtSQLSelect([u"value", u"ts"], parameter={"id":key}, dataTable=u"pool_sys", singleTable=1)
        r = db.Query(sql, values)
        if not r:
            return None
        return r[0][0]


    def LoadStructure(self, forceReload = False):
        """
        returns dictionary containing database tables and columns
        """
        if forceReload:
            self._structure = PoolStructure()
        if not self._structure.IsEmpty():
            return self._structure

        structure = {}
        fieldtypes = {}
        meta = self.GetAllMetaFlds(ignoreSystem = False)
        m = []
        f = {}
        for fld in meta:
            f[fld.id] = fld.datatype
            if fld.datatype == "file":
                continue
            m.append(fld.id)
        structure[MetaTbl] = m
        fieldtypes[MetaTbl] = f

        types = self.GetAllObjectConfs()
        for ty in types:
            t = []
            f = {}
            for fld in self.GetAllObjectFlds(ty.id):
                f[fld.id] = fld.datatype
                if fld.datatype == "file":
                    continue
                t.append(fld.id)
            structure[ty.dbparam] = t
            fieldtypes[ty.dbparam] = f
        
        self._structure.Init(structure, fieldtypes, m, self.configuration.frontendCodepage)
        # reset cached db
        self.Close()
        return structure



class AppFactory:
    """
    Internal class for dynamic object creation and caching.

    Requires:
    - Application
    """

    def _GetDataPoolObj(self):
        """
        creates the database object
        """
        poolTag = self.dbConfiguration.context
        if not poolTag:
            raise TypeError, "Database type not set. application.dbConfiguration.context is empty. Use Sqlite or Mysql!"
        elif poolTag.lower() in ("sqlite","sqlite3"):
            poolTag = "nive.utils.dataPool2.sqlite3Pool.Sqlite3"
        elif poolTag.lower() == "mysql":
            poolTag = "nive.utils.dataPool2.mySqlPool.MySql"

        # if a database connection other than the default is configured
        cTag = self.dbConfiguration.connection
        if cTag:
            connObj = GetClassRef(cTag, self.reloadExtensions, True, None)
            connObj = connObj(config=self.dbConfiguration, connectNow=False)
        else:
            connObj = None

        dbObj = GetClassRef(poolTag, self.reloadExtensions, True, None)
        conn = self.dbConfiguration
        dbObj = dbObj(connection=connObj,
                      connParam=conn,      # use the default connection defined in db if connection is none
                      structure=self._structure, 
                      root=conn.fileRoot, 
                      useTrashcan=conn.useTrashcan, 
                      dbCodePage=conn.dbCodePage,
                      debug=conn.querylog[0],
                      log=conn.querylog[1])
        return dbObj

    
    def _GetConnection(self):
        """
        Creates a new database connection. Works before startup and application setup.
        """
        if self._dbpool:
            try:
                return self._dbpool.usedconnection
            except:
                pass
                
        cTag = self.dbConfiguration.connection
        poolTag = self.dbConfiguration.context
        if cTag:
            connObj = GetClassRef(cTag, self.reloadExtensions, True, None)
            connObj = connObj(config=self.dbConfiguration, connectNow=False)
            return connObj
        if self._dbpool:
            return self._dbpool.CreateConnection(self.dbConfiguration)
        if not cTag and not poolTag:
            raise TypeError, "Database connection type not set. application.dbConfiguration.connection and application.dbConfiguration.context is empty. Use Sqlite or Mysql!"
        poolTag = self.dbConfiguration.context
        if not poolTag:
            raise TypeError, "Database type not set. application.dbConfiguration.context is empty. Use Sqlite or Mysql!"
        elif poolTag.lower() in ("sqlite","sqlite3"):
            poolTag = "nive.utils.dataPool2.sqlite3Pool.Sqlite3"
        elif poolTag.lower() == "mysql":
            poolTag = "nive.utils.dataPool2.mySqlPool.MySql"
        dbObj = GetClassRef(poolTag, self.reloadExtensions, True, None)
        return dbObj._DefaultConnection(config=self.dbConfiguration, connectNow=False)


    def _GetDBApi(self):
        """
        Creates a new and raw database connection. Works before startup and application setup.
        """
        conn = self._GetConnection()
        if conn:
            return conn.PrivateConnection()
        return None


    def _GetRootObj(self, rootConf):
        """
        creates the root object
        """
        if isinstance(rootConf, basestring):
            rootConf = self.GetRootConf(rootConf)
        if not rootConf:
            return None

        name = rootConf.id
        useCache = self.configuration.useCache
        cachename = "_c_root"+name
        if useCache and hasattr(self, cachename) and getattr(self, cachename):
            rootObj = getattr(self, cachename)
        else:
            rootObj = ClassFactory(rootConf, self.reloadExtensions, True, base=None)
            rootObj = rootObj(name, self, rootConf)
            if rootObj and useCache:
                setattr(self, cachename, rootObj)
        self.Signal("loadRoot", rootObj)
        return rootObj


    def _CloseRootObj(self, name=None):
        """
        close root objects. if name = none, all are closed.
        """
        if not name:
            n = self.GetRootIds()
        else:
            n = (name,)
        for name in n:
            cachename = "_c_root"+name
            try:
                o = getattr(self, cachename)
                o.Close()
                setattr(self, cachename, None)
            except:
                pass


    def _GetToolObj(self, name, contextObject):
        """
        creates the tool object
        """
        if isinstance(name, basestring):
            conf = self.GetToolConf(name, contextObject)
            if isinstance(conf, (list, tuple)):
                conf = conf[0]
        else:
            conf = name
        if not conf:
            iface, conf = ResolveConfiguration(name)
            if not conf:
                raise ImportError, "Tool not found. Please load the tool by referencing the tool configuration. (%s)" % (str(name))
        tag = conf.context
        toolObj = GetClassRef(tag, self.reloadExtensions, True, None)
        toolObj = toolObj(conf, self)
        if not _IGlobal.providedBy(contextObject):
            toolObj.__parent__ = contextObject
        return toolObj


    def _GetWfObj(self, wfConf, contextObject):
        """
        creates the root object
        """
        if isinstance(wfConf, basestring):
            wfConf = self.GetWorkflowConf(wfConf, contextObject)
            if isinstance(wfConf, (list, tuple)):
                wfConf = wfConf[0]
        if not wfConf:
            return None

        name = wfConf["id"]
        useCache = self.configuration.useCache
        cachename = "_c_wf"+name
        if useCache and hasattr(self, cachename) and getattr(self, cachename):
            wfObj = getattr(self, cachename)
        else:
            wfTag = wfConf["context"]
            wfObj = GetClassRef(wfTag, self.reloadExtensions, True, None)
            wfObj = wfObj(wfConf, self)
            if wfObj and useCache:
                setattr(self, cachename, wfObj)
        return wfObj


    def _CloseWfObj(self, name=None):
        """
        close wf object. if name = none, all are closed.
        """
        if not name:
            n = self.GetWorkflowIds()
        else:
            n = (name,)
        for name in n:
            cachename = "_c_wf"+name
            try:
                o = getattr(self, cachename)
                o.Close()
                setattr(self, cachename, None)
            except:
                pass





import time
import unittest
from types import DictType

from nive.definitions import *

# -----------------------------------------------------------------


class baseConfTest(unittest.TestCase):

    def test_init(self):
        c = Conf()
        c = Conf(**{"test":1})
        self.assert_(c.test==1)
        c = Conf(test=2)
        self.assert_(c.test==2)
        self.assert_(repr(c).find("<Conf ")==0)
        
    def test_set(self):
        c = Conf()
        c["test"] = 1
        self.assert_(c.test==1)
        c.test=2
        self.assert_(c.test==2)
        c.update({"test":4})
        self.assert_(c.test==4)

    def test_get(self):
        c = Conf()
        c.test=1
        self.assert_(c["test"]==1)
        self.assert_(c.get("test")==1)
        self.assert_(c.get("aaaa",3)==3)
        c.aaa=2
        c.ccc="3"
        self.assert_(c.get("ccc")=="3")
        self.assert_(c.get("aaa")==2)
        self.assert_(c.get("ooo",5)==5)
        
    def test_copy(self):
        c = Conf()
        c.test=1
        c.aaa=2
        c.bbb="4"
        c.ccc="3"
        x=c.copy()
        self.assert_(x.test==1)
        self.assert_(x.aaa==2)
        self.assert_(x.bbb=="4")
        self.assert_(x.ccc=="3")
        x=c.copy(test=5,bbb=1)
        self.assert_(x.test==5)
        self.assert_(x.aaa==2)
        self.assert_(x.bbb==1)
        self.assert_(x.ccc=="3")

    def test_copy2(self):
        c = Conf()
        c.test=1
        c.aaa=2
        c.bbb="4"
        c.ccc="3"
        x = Conf(c)
        self.assert_(x.test==1)
        self.assert_(x.aaa==2)
        self.assert_(x.bbb=="4")
        self.assert_(x.ccc=="3")
        
    def test_fncs(self):
        c = Conf(Conf(inparent=1), **{"test":1})
        c2 = c.copy(**{"test2":2})
        k=c2.keys()
        self.assert_(c2.has_key("test2"))
        self.assert_(c2.has_key("test"))
        c2.update({"test":3})
        self.assert_(c2.get("test")==3)
        self.assert_(c2.test==3)
        c2.test=4
        self.assert_(c2.inparent==1)
        self.assert_(c2.test==4)

    def test_lock(self):
        c = Conf(Conf(inparent=1), **{"test":1})
        c.test=2
        c.lock()
        self.assertRaises(ConfigurationError, c.update, {"test":4})
        self.assert_(c.test==2)
        c.unlock()
        c.test=4
        self.assert_(c.test==4)



class ConfTest(unittest.TestCase):

    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    

    def test_obj0(self, **kw):
        testconf = DatabaseConf(
            context = "Sqlite3",
            fileRoot = "",
            dbName = "",
            host = "",
            port = "",
            user = "",
            password = "",
            useTrashcan = False,
            unicode = True,
            timeout = 3,
            dbCodePage = "utf-8"
        )
        self.assert_(len(testconf.test())==0)
        self.assert_(len(testconf.uid()))
        str(testconf) # may be empty
        self.assert_(repr(testconf))

    def test_obj1(self, **kw):
        testconf = AppConf(
            id = "test",
            title = "Test",
            description = "",
            context = "nive.tests.test_definitions.ConfTest",
            fulltextIndex = False,
        )
        self.assert_(testconf.id=="test")
        self.assert_(testconf.context=="nive.tests.test_definitions.ConfTest")
        self.assert_(len(testconf.test())==0)
        self.assert_(len(testconf.uid()))
        str(testconf) # may be empty
        self.assert_(repr(testconf))

    def test_obj2(self, **kw):
        testconf = FieldConf(
            id = "aaa",
            datatype = "string",
            size = 100,
            default = "aaa",
            listItems = None,
            settings = {},
            fulltext = True,
        )
        self.assert_(testconf.id=="aaa")
        self.assert_(testconf.size==100)
        self.assert_(testconf.fulltext==True)
        self.assert_(len(testconf.test())==0)
        self.assert_(len(testconf.uid()))
        str(testconf) # may be empty
        self.assert_(repr(testconf))

    def test_obj3(self, **kw):
        testconf = ObjectConf(
            id = "text",
            name = "Text",
            dbparam = "texts",
            context = "nive.tests.test_helper.text",
            selectTag = 1,
            description = ""
        )
        self.assert_(testconf.id=="text")
        self.assert_(testconf.name=="Text")
        self.assert_(testconf.selectTag==1)
        self.assert_(len(testconf.test())==0)
        self.assert_(len(testconf.uid()))
        str(testconf) # may be empty
        self.assert_(repr(testconf))

    def test_obj4(self, **kw):
        testconf = RootConf(
            id = "root",
            name = "Root",
            context = "nive.tests.test_definitions.ConfTest",
            default = True
        )
        self.assert_(testconf.id=="root")
        self.assert_(testconf.name=="Root")
        self.assert_(testconf.default==True)
        self.assert_(len(testconf.test())==0)
        self.assert_(len(testconf.uid()))
        str(testconf) # may be empty
        self.assert_(repr(testconf))

    def test_obj5(self, **kw):
        testconf = ViewModuleConf(
            id = u"viewing",
            name = u"Oh",
            static = "here:static",
            containment = "nive.tests.test_definitions.ConfTest",
            widgets = (WidgetConf(apply=(IObject,),viewmapper="test",widgetType=IWidgetConf,id="test"),),
        )
        self.assert_(testconf.id=="viewing")
        self.assert_(testconf.name=="Oh")
        self.assert_(testconf.containment=="nive.tests.test_definitions.ConfTest")
        self.assert_(len(testconf.test())==0)
        self.assert_(len(testconf.uid()))
        str(testconf) # may be empty
        self.assert_(repr(testconf))

    def test_obj6(self, **kw):
        testconf = ViewConf(
            attr = "test_obj6",
            name = "",
            context = "nive.tests.test_definitions.SystemTest",
            view = "nive.tests.test_definitions.ConfTest",
            renderer = None
        )
        self.assert_(testconf.attr=="test_obj6")
        self.assert_(testconf.name=="")
        self.assert_(testconf.view=="nive.tests.test_definitions.ConfTest")
        self.assert_(len(testconf.test())==0)
        self.assert_(len(testconf.uid()))
        str(testconf) # may be empty
        self.assert_(repr(testconf))

    def test_obj7(self, **kw):
        testconf = ToolConf(
            id = u"tool",
            name = u"Tool",
            context = "nive.tests.test_definitions.ConfTest",
        )
        self.assert_(testconf.id=="tool")
        self.assert_(testconf.name=="Tool")
        self.assert_(testconf.context=="nive.tests.test_definitions.ConfTest")
        self.assert_(len(testconf.test())==0)
        self.assert_(len(testconf.uid()))
        str(testconf) # may be empty
        self.assert_(repr(testconf))
    
    def test_obj8(self, **kw):
        testconf = ModuleConf(
            id = "module",
            name = "Module",
            context = "nive.tests.test_definitions.ConfTest",
            events = None,
            description = ""
        )
        self.assert_(len(testconf.test())==0)
        self.assert_(len(testconf.uid()))
        str(testconf) # may be empty
        self.assert_(repr(testconf))
    
    def test_obj9(self, **kw):
        testconf = WidgetConf(
            apply = (IObject,IApplication),
            viewmapper = "viewme",
            widgetType = IWidgetConf,
            name = "widget",
            id = "widget",
            sort = 100,
            description = ""
        )
        self.assert_(testconf.id=="widget")
        self.assert_(testconf.name=="widget")
        self.assert_(len(testconf.test())==0)
        self.assert_(len(testconf.uid()))
        str(testconf) # may be empty
        self.assert_(repr(testconf))
    
    def test_obj10(self, **kw):
        testconf = GroupConf(
            id = "group",
            name = "Group",
            hidden = True,
            description = ""
        )
        self.assert_(testconf.id=="group")
        self.assert_(testconf.name=="Group")
        self.assert_(testconf.hidden==True)
        self.assert_(len(testconf.uid()))
        str(testconf) # may be empty
        self.assert_(repr(testconf))
    
    def test_obj11(self, **kw):
        testconf = CategoryConf(
            id = "category",
            name = "Category",
            hidden = True,
            description = ""
        )
        self.assert_(testconf.id=="category")
        self.assert_(testconf.name=="Category")
        self.assert_(testconf.hidden==True)
        self.assert_(len(testconf.uid()))
        str(testconf) # may be empty
        self.assert_(repr(testconf))
    
    def test_obj12(self, **kw):
        testconf = PortalConf(
            portalDefaultUrl = "/website/",
            loginUrl = "/userdb/udb/login",
            forbiddenUrl = "/userdb/udb/login",
            logoutUrl = "/userdb/udb/logout",
            accountUrl = "/userdb/udb/update",
            favicon = "",
            robots = u""
        )
        self.assert_(testconf.portalDefaultUrl=="/website/")
        self.assert_(testconf.loginUrl == "/userdb/udb/login")
        self.assert_(testconf.forbiddenUrl == "/userdb/udb/login")
        self.assert_(testconf.logoutUrl == "/userdb/udb/logout")
        self.assert_(testconf.accountUrl == "/userdb/udb/update")
        self.assert_(testconf.robots == u"")
        self.assert_(len(testconf.uid()))
        str(testconf) # may be empty
        self.assert_(repr(testconf))
    
        
        
class SystemTest(unittest.TestCase):

    def test_data(self, **kw):
        self.assert_(len(DataTypes))
        self.assert_(len(SystemFlds))
        self.assert_(len(ReadonlySystemFlds))
        

    def test_structure1(self, **kw):
        for tbl in Structure.items():
            self.assert_(tbl[0])
            self.assert_("fields" in tbl[1])
        


if __name__ == '__main__':
    unittest.main()



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
Basic view class for nive object views. 

Use this class as base for custom view definitions for file download support,
data rendering, url generation, http headers and user lookup. 
"""


import os
import time
from datetime import datetime
from email.utils import formatdate

from pyramid.response import Response
from pyramid.renderers import render_to_response, get_renderer, render
from pyramid.url import static_url, resource_url
from pyramid.view import render_view
from pyramid.security import authenticated_userid
from pyramid.security import has_permission
from pyramid.i18n import get_localizer

from pyramid.httpexceptions import HTTPNotFound, HTTPFound, HTTPOk, HTTPForbidden
from pyramid.exceptions import NotFound

from nive.i18n import _
from nive.utils.utils import ConvertToStr, ConvertListToStr, ConvertToDateTime
from nive.utils.utils import FmtSeconds, FormatBytesForDisplay, CutText, GetMimeTypeExtension
from nive import FileNotFound
from nive.definitions import IPage, IObject



class Unauthorized(Exception):
    """
    Raised by failed logins or insufficient permissions
    """
    


class BaseView(object):
    """    """
    
    def __init__(self, context, request):
        self.context = context
        self.request = request
        self.viewModule = None
        self.appRequestKeys = []
        self._t = time.time()
        self.fileExpires = 3600


    # url handling ----------------------------------------------------------------

    def Url(self, resource=None):
        """
        Generates the default url for the resource. Includes the file extension if one specified 
        for the object.
        
        If resource is None the current context object is used as resource.

        returns url
        """
        if not resource:
            resource=self.context
        if hasattr(resource, "extension") and resource.extension:
            return u"%s.%s" % (resource_url(resource, self.request)[:-1], resource.extension)
        return resource_url(resource, self.request)

    def FolderUrl(self, resource=None):
        """
        Generates the default url for the resource without extension and trailing '/'. 
        
        If resource is None the current context object is used as resource.

        returns url
        """
        if not resource:
            resource=self.context
        return resource_url(resource, self.request)

    def StaticUrl(self, file):
        """
        Generates the url for a static file.
        *file* is the filename to look the url up for. E.g. for the default static directory use ::
        
            <link tal:attributes="href view.StaticUrl('layout2.css')" 
                  rel="stylesheet" type="text/css" media="all" />
                  
         to reference a file in a different directory you must include the python module like ::

            <link tal:attributes="href view.StaticUrl('nive.cms.design:static/layout2.css')" 
                  rel="stylesheet" type="text/css" media="all" />
                  
        returns url
        """
        if not u":" in file and self.viewModule and self.viewModule.static:
            file = u"%s/%s" % (self.viewModule.static, file)
        return static_url(file, self.request)

    def FileUrl(self, fieldID, resource=None):
        """
        Generates the file url for the file contained in resource. The resource must have the 'file'
        view included (IFile). If the url is called the download is mapped to ``View.file``. 
        
        If resource is None the current context object is used as resource.

        returns url
        """
        if not resource:
            resource=self.context
        file = resource.files.get(fieldID)
        if not file:
            return u""
        return u"%sfile/%s" % (self.Url(resource), file.filename)

    def PageUrl(self, resource=None, usePageLink=0):
        """
        Generates the default page url for the resource with extension. If resource is a page element
        the page containing this element is used for the url. 
        
        If resource is None the current context object is used as resource.

        returns url
        """
        if not resource:
            resource=self.context
        try:
            page = resource.GetPage()
        except:
            page = self.context
        link = page.data.get("pagelink")
        if usePageLink and link:
            return self.ResolveUrl(link, resource)
        if hasattr(page, "extension"):
            return u"%s.%s" % (resource_url(page, self.request)[:-1], page.extension)
        return resource_url(page, self.request)

    def CurrentUrl(self, retainUrlParams=False):
        """
        Returns the current url that triggered this request. Url parameter are removed by
        default.
        
        returns url
        """
        if retainUrlParams:
            return self.request.url
        return self.request.url.split(u"?")[0]

    def ResolveUrl(self, url, object=None):
        """
        Resolve a string to url for object or the current context.

        Possible values:
        
        - page_url
        - obj_url
        - obj_folder_url
        - parent_url
        """
        if url==None:
            return u""
        if not object or not IObject.providedBy(object):
            object = self.context
        if url == "page_url":
            url = self.PageUrl(object)
        elif url == "obj_url":
            url = self.Url(object)
        elif url.find("obj_folder_url")!=-1:
            url = url.replace("obj_folder_url", self.FolderUrl(object))
        elif url == "parent_url":
            url = self.Url(object.parent)
        return url
    
    
    def ResolveLink(self, link):
        """
        Resolve the link to a valid (page) URL. If the link is an existing object.id the url to the 
        page containing this object is returned.
        
        returns url
        """
        try:
            i = int(link)
            o = self.context.GetRoot().LookupObj(i)
            if not o:
                return link
            try:
                return self.PageUrl(o)
            except:
                return self.Url(o)
        except:
            return link


    def SendResponse(self, data, mime="text/html", raiseException=False, filename=None):
        """
        Creates a response with data as body. If ``raiseException`` is true the function
        will raise a HTTPOk Exception with data as body.
        
        If filename is not none the response will extended with a ``attachment; filename=filename``
        header.
        """
        cd = None
        if filename:
            cd = 'attachment; filename=%s'%(filename)
        if raiseException:
            raise HTTPOk(content_type=mime, body=data, content_disposition=cd)
        return Response(content_type=mime, body=data, content_disposition=cd)
        
        
    def Redirect(self, url, messages=None, slot=""):
        """
        Redirect to the given URL. Messages are stored in session and can be accessed
        by calling ``request.session.pop_flash()``. Messages are added by calling
        ``request.session.flash(m, slot)``.
        """
        if messages:
            if isinstance(messages, basestring):
                self.request.session.flash(messages, slot)
            else:
                for m in messages:
                    self.request.session.flash(m, slot)
        headers = None
        if hasattr(self.request.response, "headerlist"):
            headers = self.request.response.headerlist
        raise HTTPFound(location=url, headers=headers)
    

    def Relocate(self, url, messages=None, slot="", raiseException=False):
        """
        Returns messages and X-Relocate header in response.
        If raiseException is True HTTPOk is raised with empty body.
        """
        if messages:
            if isinstance(messages, basestring):
                self.request.session.flash(messages, slot)
            else:
                for m in messages:
                    self.request.session.flash(m, slot)
        headers = [('X-Relocate', str(url))]
        if hasattr(self.request.response, "headerlist"):
            headers += list(self.request.response.headerlist)
        self.request.response.headerlist = headers    
        if raiseException:
            raise HTTPOk(headers=headers, body_template="redirect "+url)
        return u""

    def Relocated(self):
        return u""

    def ResetFlashMessages(self, slot=""):
        """
        Removes all messages stored in session.
        """
        while self.request.session.pop_flash(slot):
            continue
        
    
    def AddHeader(self, name, value):
        """
        Add a additional response header value.
        """
        headers = [(name, value)]
        if hasattr(self.request.response, "headerlist"):
            headers += list(self.request.response.headerlist)
        self.request.response.headerlist = headers    

    
    # render other elements and objects ---------------------------------------------
    
    def index_tmpl(self, path=None):
        if path:
            return get_renderer(path).implementation()
        if not self.viewModule or not self.viewModule.mainTemplate:
            return None
        return get_renderer(self.viewModule.mainTemplate).implementation()
    

    def DefaultTemplateRenderer(self, values, templatename = None):
        """
        Renders the default template configured in context.configuration.template 
        with the given dictionary `values`. Calls `CacheHeader` to set the default 
        cache headers.
        
        Adds the following values if not set ::
            
            {u"context": self.context, u"view": self} 
        
        Template lookup first searches the current view module and if not found
        the parent or extended view module. 
        """
        if not templatename:
            templatename = self.context.configuration.template
            if not templatename:
                templatename = self.context.configuration.id
        tmpl = self._LookupTemplate(templatename)
        if not tmpl:
            raise ConfigurationError, "Template not found: %(name)s %(type)s." % {"name": templatename, "type": self.context.configuration.id}
        if not "context" in values: values[u"context"] = self.context
        if not "view" in values: values[u"view"] = self
        response = render_to_response(tmpl, values, request=self.request)
        self.CacheHeader(response, user=self.User())
        return response
            

    def RenderView(self, obj, name="", secure=True, raiseUnauthorized=False, codepage="utf-8"):
        """
        Render a view for the object.
        
        *name* is the name of the view to be looked up.
        
        If *secure* is true permissions of the current user are checked against the view. If the
        user lacks the necessary permissions and empty string is returned or if *raiseUnauthorized*
        is True HTTPForbidden is raised. 
        
        returns rendered result
        """
        # store original context to reset after calling render_view
        orgctx = self.request.context
        self.request.context = obj
        if not raiseUnauthorized:
            try:
                value = render_view(obj, self.request, name, secure)
                self.request.context = orgctx
            except HTTPForbidden:
                self.request.context = orgctx
                return u""
        else:
            try:
                value = render_view(obj, self.request, name, secure)
            except:
                self.request.context = orgctx
                raise
        self.request.context = orgctx
        if not value:
            return u""
        return unicode(value, codepage)

    
    def IsPage(self, object=None):
        """
        Check if object is a page.
        
        returns bool
        """
        if not object:
            return IPage.providedBy(self.context)
        return IPage.providedBy(object)
    
    def tmpl(self):
        """
        Default function for template rendering.
        """
        return {}
    
    
    # file handling ------------------------------------------------------------------
    
    def file(self):
        return self.File()

    def File(self):
        """
        Used by "file" view for the current context. 
        
        Calls SendFile() for the file matching the filename of the current url.
        """
        if not len(self.request.subpath):
            raise NotFound
        file = self.context.GetFileByName(self.request.subpath[0])
        if not file:
            raise NotFound
        return self.SendFile(file)


    def SendFile(self, file):
        """
        Creates the response and sends the file back. Uses the FileIterator.
        
        #!date format
        """
        if not file:
            return HTTPNotFound()
        last_mod = file.mtime()
        if not last_mod:
            last_mod = self.context.meta.pool_change
        r = Response(content_type=str(GetMimeTypeExtension(file.extension)), conditional_response=True)
        iterator = file.iterator()
        if iterator:
            r.app_iter = iterator
        else:
            try:
                r.body = file.read()
            except FileNotFound:
                raise NotFound
        r.content_length = file.size
        r.last_modified = last_mod
        r.etag = '%s-%s' % (last_mod, hash(file.path))
        r.cache_expires(self.fileExpires)
        return r    


    # http caching ----------------------------------------------------------------

    def CacheHeader(self, response, user=None):
        """
        Adds a http cache header to the response. If user is not None *NoCache()* is called, otherwise
        *Modified()*.
        
        returns response
        """
        if user:
            return self.NoCache(response)
        return self.Modified(response, user)

    def NoCache(self, response, user=None):
        """
        Adds a no-cache header to the response.
        
        returns response
        """
        response.cache_control = u"no-cache"
        response.etag = '%s-%s' % (response.content_length, str(self.context.id))
        return response

    def Modified(self, response, user=None):
        """
        Adds a last modified header to the response. meta.pool_change is used as date.

        returns response
        """
        if user:
            response.last_modified = formatdate(timeval=None, localtime=True, usegmt=True)
        else:
            if self.context.meta.get("pool_change"):
                t = ConvertToDateTime(self.context.meta.get("pool_change")).timetuple()
                t = time.mktime(t)
            else:
                t = None
            response.last_modified = formatdate(timeval=t, localtime=True, usegmt=True)
        response.etag = '%s-%s-%s' % (response.last_modified, response.content_length, str(self.context.id))
        return response


    # user and security -------------------------------------------------
    
    @property
    def user(self):
        return self.User()
    
    def User(self, sessionuser=True):
        """
        Get the currently signed in user. If sessionuser=False the function will return
        the uncached write enabled user from database.
        
        returns the *Authenticated User Object* or None
        """
        # cached session user object
        if not sessionuser:
            ident = authenticated_userid(self.request)
            if not ident:
                return None
            return self.context.app.portal.userdb.root().LookupUser(ident=ident)
        try:
            user = self.request.environ["authenticated_user"]
            if user:
                return user
        except KeyError:
            pass
        ident = authenticated_userid(self.request)
        if not ident:
            return None
        return self.context.app.portal.userdb.root().GetUser(ident)
    
    def UserName(self):
        """
        returns the *Authenticated User Name* or None
        """
        return authenticated_userid(self.request)    

    def Allowed(self, permission, context=None):
        """
        Check if the current user has the *permission* for the *context*.
        If context is none the current context is used
        
        returns True / False
        """
        if not context:
            context = self.context
        return has_permission(permission, context, self.request)
    
    def InGroups(self, groups):
        """
        Check if current user is in one of the groups.
        """
        user = self.User()
        if not user:
            return False
        return user.InGroups(groups)
    

    # render helper ---------------------------------------------------------
    
    def GetLocale(self):
        if hasattr(self, "_c_locale"):
            return self._c_locale
        l = u"en"
        self._c_locale = l
        return l
    
    def GetTimezone(self):
        if hasattr(self, "_c_tz"):
            return self._c_tz
        l = u"GMT"
        self._c_tz = l
        return l
    
    def RenderField(self, fld, data=None):
        """
        Render the data field for html display. Rendering depends on the datatype defined
        in the field configuration.
        
        If data is None the current context is used.
        
        returns string
        """
        if isinstance(fld, basestring):
            fld = self.context.GetFieldConf(fld)
        if not fld:
            return _(u"<em>Unknown field</em>")
        if data == None:
            data = self.context.data.get(fld['id'], self.context.meta.get(fld['id']))
        if fld['datatype']=='file':
            url = self.FileUrl(fld['id'])
            if not url:
                return u""
            url2 = url.lower()
            if url2.find(u".jpg")!=-1 or url2.find(u".jpeg")!=-1 or url2.find(u".png")!=-1 or url2.find(u".gif")!=-1:
                return u"""<img src="%s" />""" % (url)
            return u"""<a href="%s">download</a>""" % (url)
        return FieldRenderer(self.context).Render(fld, data, context=self.context)
    
    def HtmlTitle(self):
        t = self.request.environ.get(u"htmltitle")
        if t:
            return t
        return self.context.GetTitle()
    
    
    def Translate(self, text):
        """
        Tranlate a translation string. Extracts language from request.
        """
        localizer = get_localizer(self.request)
        return localizer.translate(text)
        
    
    def FmtTextAsHTML(self, text):
        """
        Converts newlines to <br/>.
        
        returns string
        """
        if not text:
            return u""
        if text.find(u"\r\n")!=-1:
            text = text.replace(u"\r\n",u"<br/>\r\n")
        else:
            text = text.replace(u"\n",u"<br/>\n")
        return text

    def FmtDateText(self, date, language=None):
        """
        Formats dates as readable text in conventional format.
        
        returns string
        """
        if not date:
            return u""
        if not isinstance(date, datetime):
            date = ConvertToDateTime(date)
        return date.strftime(u"%c")

    def FmtDateNumbers(self, date, language=None):
        """
        Formats dates as numbers e.g 13.12.2011.
        
        returns string
        """
        if not date:
            return u""
        if not isinstance(date, datetime):
            date = ConvertToDateTime(date)
        return date.strftime(u"%x")
    
    def FmtSeconds(self, secs):
        """ seconds to readable text """
        return FmtSeconds(secs)

    def FmtBytes(self, size):
        """ bytes to readable text """
        return FormatBytesForDisplay(size)

    def CutText(self, text, length):
        """ bytes to readable text """
        return CutText(text, length)


    # parameter handling ------------------------------------------------------------

    def GetFormValue(self, key, default=None, request=None, method=None):
        """
        Extract a form field from request. 
        Works with webob requests and simple dictionaries.
        
        returns the value or *default*
        """
        if not request:
            request = self.request
        try:
            if method == "POST":
                value = request.POST.getall(key)
            elif method == "GET":
                value = request.GET.getall(key)
            else:
                value = request.POST.getall(key)
                if not value:
                    value = request.GET.getall(key)
            if isinstance(value, bytes):
                value = unicode(value, self.context.app.configuration.frontendCodepage)
        except (AttributeError,KeyError):
            if method == "POST":
                value = request.POST.get(key)
            elif method == "GET":
                value = request.GET.get(key)
            else:
                value = request.POST.get(key)
                if not value:
                    value = request.GET.get(key)
            if isinstance(value, bytes):
                value = unicode(value, self.context.app.configuration.frontendCodepage)
            if value==None:
                return default
            return value
        if not len(value):
            return default
        elif len(value)==1:
            if value==None:
                return default
            return value[0]
        if value==None:
            return default
        return value            


    def GetFormValues(self, request=None, method=None):
        """
        Extract all form fields from request. 
        Works with webob requests and simple dictionaries.
        
        returns dictionary
        """
        if not request:
            request = self.request
        try:
            if method == "POST":
                values = request.POST.mixed()
            elif method == "GET":
                values = request.GET.mixed()
            else:
                values = request.GET.mixed()
                values.update(request.POST.mixed())
        except AttributeError:
            try:
                if method == "POST":
                    values = request.POST
                elif method == "GET":
                    values = request.GET
                else:
                    values = request.GET
                    values.update(request.POST)
            except AttributeError:
                values = request
        return values    
    
    def FmtURLParam(self, **kw):
        """
        Format all kw items as url parameter. 
        
        returns string
        """
        url = []
        params = kw
        #opt
        for p in params.keys():
            if len(url):
                url.append(u"&")
            url.append(u"%s=%s" % (p, params[p]))
        return u"".join(url)    #urllib.quote_plus()
    

    def FmtFormParam(self, **kw):
        """    
        Format all kw items as hidden input fields for forms.
        
        returns string 
        """
        form = []
        params = kw
        for p in params.keys():
            value = ConvertToStr(params[p])
            form.append(u"<input type='hidden' name='%s' value='%s' />" % (p, value))
        return "".join(form)


    def _LookupTemplate(self, tmplfile):
        if not self.viewModule or u":" in tmplfile:
            return tmplfile
        fn = self.viewModule.templates + u"/" + tmplfile
        try:
            if get_renderer(fn):
                return fn
        except ValueError:
            pass
        if not self.viewModule.parent:
            return tmplfile
        return u"%s/%s" % (self.viewModule.parent.templates, tmplfile)


    def mark(self):
        return time.time() - self._t

    def mark2(self):
        try:
            return u"""<div class="mark">%.04f</div>""" % (time.time() - self.request.environ.get("START_TIME", self._t))
        except:
            return u""


    # bw 0.9.7 ----------------------------------------------------------------------------
    def AjaxRelocate(self, url, messages=None, slot="", raiseException=False):
        return self.Relocate(url, messages=messages, slot=slot, raiseException=raiseException)
 




class FieldRenderer(object):
    
    def __init__(self, context, skip=()):
        self.context = context
        self.skipRender = skip
        
    def Render(self, fieldConf, value, useDefault=False, listItems=None, context=None, **kw):
        """
        fieldConf = FieldConf of field to be rendered
        value = the value to be rendered
        useDefault = use default values from fieldConf if not found in value
        listItems = used for list lookup key -> name
        context = context object the field is rendered for. Required if listItems=None.
        
        **kw:
        static = static root path for images
        """
        data = ""
        if useDefault:
            data = fieldConf["default"]
        if value != None:
            data = value
        if fieldConf.id in self.skipRender:
            return data

        def loadListItems(fld, context):
            if not context:
                return []
            pool_type = context.GetTypeID() 
            return context.root().LoadListItems(fld, obj=context, pool_type=pool_type)
        
        # format for output
        fType = fieldConf["datatype"]

        # fomat settings
        fmt = fieldConf.settings.get("format")
        if fmt:
            if fmt=="bytesize":
                data = FormatBytesForDisplay(data)
                return data
            elif fmt=="image":
                tmpl = fieldConf.settings.get("path", u"")
                path = tmpl % {"data":data, "static": kw.get("static",u"")}
                data = """<img src="%(path)s" title="%(name)s" />""" % {"path":path, "name": fieldConf.name}
                return data
        
        if fType == "bool":
            if data:
                data = _(u"Yes")
            else:
                data = _(u"No")

        elif fType == "string":
            if fieldConf.settings.get("relation") == u"userid":
                # load user name from database
                try:
                    udb = context.app.portal.userdb.root()
                    user = udb.GetUser(data, activeOnly=0)
                    if user:
                        data = user.GetTitle()
                except AttributeError:
                    pass

        elif fType == "text":
            data = data.replace(u"\r\n", u"\r\n<br />")

        elif fType == "date":
            if not isinstance(data, datetime):
                if not data:
                    return u""
                data = ConvertToDateTime(data)
            fmt = fieldConf.settings.get("strftime", u"%x")
            return data.strftime(fmt)

        elif fType in ("datetime", "timestamp"):
            fmt = fieldConf.settings.get("format")
            if not isinstance(data, datetime):
                if not data:
                    return u""
                data = ConvertToDateTime(data)
            # defaults
            fmt = fieldConf.settings.get("strftime")
            if not fmt:
                fmt = u"%x %H:%M"
                # hide hour and minutes if zero
                if data.hour==0 and data.minute==0 and data.second==0:
                    fmt = u"%x"
                elif fieldConf.settings.get("seconds"):
                    fmt = u"%x %X"
            return data.strftime(fmt)

        elif fType == "unit":
            if data == 0 or data == "0":
                return ""
            return data

        elif fType == "unitlist":
            if(type(data) == ListType):
                return ConvertListToStr(data)
            return str(data)

        elif fType == "list" or fType == "radio":
            options = listItems or loadListItems(fieldConf, context)
            if not options:
                options = fieldConf.get("listItems")
                if hasattr(options, "__call__"):
                    options = options(fieldConf, self.context)

            if options:
                for item in options:
                    if item["id"] == data:
                        data = item["name"]
                        break
                
        elif fType == "mselection" or fType == "mcheckboxes":
            values = []
            options = listItems or loadListItems(fieldConf, context)
            if not options:
                options = fieldConf.get("listItems")
                if hasattr(options, "__call__"):
                    options = options(fieldConf, self.context)

            if isinstance(data, basestring):
                data = tuple(value.split(u"\n"))
            for ref in data:
                if options:
                    for item in options:
                        if item["id"] == ref:
                            values.append(item["name"])
                else:
                    values.append(ref)
                
            data = u", ".join(values)

        elif fType == "url":
            if render:
                if data != u"" and data.find(u"http://") == -1:
                    data = u"http://" + data
                l = data[7:50]
                if len(data) > 50:
                    l += u"..."
                data = u"<a alt='%s' href='%s' target='_blank'>%s</a>" % (data, data, l)

        elif fType == "urllist":
            urllist = data
            data = []
            if len(urllist) and urllist[0] == u"[":
                urllist = urllist[1:-1]
                ul = urllist.split(u"', u'")
                ul[0] = ul[0][1:]
                ul[len(ul)-1] = ul[len(ul)-1][:-1]
            else:
                ul = urllist.replace(u"\r",u"")
                ul = ul.split(u"\n")
            links = []
            for l in ul:
                t = l.split(u"|||")
                title = u""
                url = t[0]
                if len(t) > 1:
                    title = t[1]
                if title != u"":
                    title = u'%s (%s)' % (title, url)
                else:
                    title = url
                links.append({"id": l, "name": title})
                data.append(u"<a alt='%s' title='%s' href='%s' target='_blank'>%s</a><br/>" % (title, title, url, title))
            data = u"".join(data)

        elif fType == "password":
            data = u"*****"

        return data
    

# Mail template mapper -------------------------------------------------

class Mail(object):
    """
    Mail template object with template link and title. ::
    
        mail = Mail(title="New mail", tmpl="mypackage:templates/foo.pt")
        body = mail(value1=1, value2=2)
        
    the body of the mail is generated by calling the mail object. *kw* parameters
    are passed on in template.render()
    """
    title = u""
    tmpl = None     # 'mypackage:templates/foo.pt'
    
    def __init__(self, title=u"", tmpl=None):
        self.title = title
        self.tmpl = tmpl
    
    def __call__(self, **kws):
        mail = render(self.tmpl, kws)
        return mail


# Configuration -------------------------------------------------
"""
???
"""
def Redirect(url, request, messages=None):
    headers = None
    if hasattr(request.response, "headerlist"):
        headers = request.response.headerlist
    return HTTPFound(location=url, headers=headers)

def AuthenticatedUserName(request):
    # bw 0.9.6. removed in next version.
    return authenticated_userid(request)

def forbidden_view(request):
    return Response('forbidden')





#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
Extensions for easier Page, Column and Page element handling.
"""

import string
from datetime import datetime

from nive.utils.utils import ConvertToList
from nive.definitions import StagPage, StagPageElement
from nive.security import Allow, Deny, Authenticated, Everyone


class PageContainer:
    """
    Extension for page containers. Adds sub page selection functionality.
    """

    def GetPages(self, includeMenu=1, hidden=1, public=None):
        """
        Returns sub pages as objects based on parameters. ::
        
            includeMenu = include pages marked as menus
            hidden = include hidden pages
            public = only published pages (pool_state=1). if none 
                     queryRestraint default is used
            
        returns object list
        """
        parameter, operators = {},{}
        parameter[u"pool_stag"] = (StagPage, StagPageElement-1)
        operators[u"pool_stag"] = u"BETWEEN"
        if public==1:
            parameter[u"pool_state"] = 1
        elif public==0:
            if u"pool_state" in parameter:
                del parameter[u"pool_state"]
        pages = self.GetObjs(parameter = parameter, operators = operators)
        if includeMenu and hidden:
            return pages
        p2 = []
        for p in pages:
            if not includeMenu and p.data["navMenu"]:
                continue
            if not hidden and p.data["navHidden"]:
                continue
            p2.append(p)
        return p2


    def GetPagesList(self, includeMenu=1, hidden=1, public=None, fields=None):
        """
        Returns sub pages as objects based on parameters. ::
        
            includeMenu = include pages marked as menus
            hidden = include hidden pages
            public = only published pages (pool_state=1). if none 
                     queryRestraint default is used
            fields = fields names included in result
            
        returns object list
        """
        if not fields:
            fields = [u"title", u"header", u"description", u"keywords",
                      u"navMenu", u"navHidden",
                      u"pool_state", u"pool_wfa", u"pool_unitref", u"pool_stag", u"pool_filename"]
        parameter, operators = {},{}
        parameter[u"pool_stag"] = (StagPage, StagPage+9)
        operators[u"pool_stag"] = u"BETWEEN"
        parameter[u"pool_unitref"] = self.id
        if public==1:
            parameter[u"pool_state"] = 1
        elif public==0:
            if u"pool_state" in parameter:
                del parameter[u"pool_state"]
        if not hidden:
            parameter[u"navHidden"] = 0
        if not includeMenu:
            parameter[u"navMenu"] = 0
        sort = self.GetSort()
        root = self.root()
        parameter,operators = root.ObjQueryRestraints(self, parameter, operators)
        pages = root.SelectDict(pool_type=u"page", parameter=parameter, fields=fields, operators=operators, sort=sort, dontAddType=True)
        return pages


    def GetElementsList(self, fields=None):
        """
        Returns sub pages as objects based on parameters. ::
        
            fields = fields names included in result
            
        returns object list
        """
        if not fields:
            fields = [u"id", u"type", u"pool_filename", u"pool_unitref", u"pool_stag"]
        parameter, operators = {},{}
        parameter[u"pool_stag"] = (StagPageElement, StagPageElement+9)
        operators[u"pool_stag"] = u"BETWEEN"
        parameter[u"pool_unitref"] = self.id
        sort = self.GetSort()
        root = self.root()
        parameter,operators = root.ObjQueryRestraints(self, parameter, operators)
        pages = root.SelectDict(pool_type=u"page", parameter=parameter, fields=fields, operators=operators, sort=sort, dontAddType=True)
        return pages


class PageElementContainer:
    """
    Adds page element container functions to the object.
    """

    # Contained page elements/editor -------------------------------------------------------------

    @property
    def page(self):
        return self

    def IsEmpty(self):
        """
        Check for empty page with no elements on it
        
        returns True/False
        """
        return len(self.GetPageElements()) == 0


    def GetPageElements(self, addBoxContents=0, addColumnContents=0, **kw):
        """
        List all contained elements. ::
        
            addBoxContents = add elements contained in boxes
            skipColumns = do not include columns
            kw = passed to GetObj() internally
            
        returns a list of elements
        """
        # b.w. 0.9.9
        if kw.get("skipColumns")!=None:
            addColumnContents = not bool(kw.get("skipColumns"))
        parameter, operators = {},{}
        parameter[u"pool_stag"] = StagPageElement
        if not addColumnContents:
            parameter[u"pool_type"] = u"column"
            operators[u"pool_type"] = u"!="
        elements = self.GetObjs(parameter = parameter, operators = operators, **kw)
        if not addBoxContents and not addColumnContents:
            return elements
        elements2 = []
        for e in elements:
            t = e.GetTypeID()
            if addBoxContents and t == "box":
                elements2 += e.GetPageElements(addBoxContents, addColumnContents, **kw)
            elif addColumnContents and t == "column":
                elements2 += e.GetPageElements(addBoxContents, addColumnContents, **kw)
        return elements + elements2


    def AllowedPageElements(self, user, group=None):
        """
        Return a list of page element configurations allowed to be added in
        this container
        """
        elements = []
        for t in self.app.GetAllObjectConfs():
            if t["selectTag"] == StagPageElement and not t.get("hidden"):
                elements.append(t)
        return elements


    def GetElementContainer(self):
        """ return the current element container """
        return self

    def GetPage(self):
        """ return the current page """
        return self



class PageElement:
    """
    Adds page element functionality to objects.
    
    Sets local acl 's based on object.meta.pool_group settings.
    """
    
    @property
    def page(self):
        return self.parent.page

    def Init(self):
        self.ListenEvent("commit", self.TouchPage)
        groups = self.meta.get("pool_groups")
        if groups:
            groups = ConvertToList(groups)
            acl = [(Allow, "group:editor", "view"),(Allow, "group:author", "view"),(Allow, "group:admin", "view")]
            for g in groups:
                if g == u"authenticated":
                    acl.append((Allow, Authenticated, "view"))
                else:
                    acl.append((Allow, g, "view"))
            acl.append((Deny, Everyone, 'view'))
            self.__acl__ = acl 


    def GetElementContainer(self):
        """ Returns the current element container """
        return self.GetParent()


    def GetPage(self):
        """ Return the current page """
        return self.GetParent().GetPage()
    
    
    def TouchPage(self, user=None):
        """ Changes the change date and time of the containing page to now """
        try:
            # calls touch for the page
            self.page.CommitInternal(user=user) 
        except AttributeError:
            self.page.meta[u"pool_change"]=datetime.now()
    

class PageColumns:
    """
    Column support for pages.
    
    Columns are always referenced by *name* and only one column for each name 
    is created locally. *Local* means columns may be loaded from parent pages
    if not existing in this object.
    
    Checks whether a column already exists before it is created.
    """
    
    def Init(self):
        self.ListenEvent("beforeAdd", "_CheckColumn")

    
    def GetColumn(self, name):
        """
        Returns the column *name* as object. If no local column is found, parents 
        searched.  
        """
        c = self.LocalColumn(name)
        if c:
            return c
        parent = self.GetParent()
        while parent:
            c = parent.LocalColumn(name)
            if c:
                return c
            if parent.GetID() == 0:
                break
            parent = parent.GetParent()
        return None
        

    def GetProxiedColumn(self, name):
        """
        Returns the column *name* as object. If no local column is found, parents 
        searched.  
        """
        c = self.LocalColumn(name)
        if c:
            return c
        parent = self.GetParent()
        while parent:
            c = parent.LocalColumn(name)
            if c:
                c.__parent__ = self
                return c
            if parent.GetID() == 0:
                break
            parent = parent.GetParent()
        return None
        

    def HasLocalColumn(self, name):
        """
        Returns if a local column *name* exists. 
        """
        parameter = {u"pool_type": u"column", u"title": name}
        operators = {u"pool_type": u"=", u"title": u"="}
        columns = self.GetObjsList(fields = [u"id"], parameter=parameter, operators=operators)
        if not len(columns):
            return False
        return True

        
    def LocalColumn(self, name):
        """
        Returns the local column *name* as object or None.
        """
        parameter = {u"pool_type": u"column", u"title": name}
        operators = {u"pool_type": u"=", u"title": u"="}
        columns = self.GetObjs(parameter=parameter, operators=operators)
        if not len(columns):
            return None
        return columns[0]
        
    
    def _CheckColumn(self, data, type, **kw):    
        """
        Checks if new column is going to be created and if already one exists
        """
        if type != u"column":
            return
        name = data.get("title")
        c = self.LocalColumn(name)
        if not c:
            return
        raise TypeError, "Column exists (%s)" % (name)
    



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

import string

from nive.i18n import _
from nive.utils.utils import ConvertListToStr, ConvertToNumberList

from nive.definitions import StagPage, StagPageElement
from nive.definitions import IContainer, ViewConf

from nive.cms.cmsview.sort import ISort






class ObjCopy:
    """
    Cut, copy and paste functionality for objects 
    """
    
    def CanCopy(self):
        """
        """
        return not hasattr(self, "disableCopy") or not self.disableCopy


    def CanPaste(self):
        """
        """
        return False



class ContainerCopy:
    """
    Cut, copy and paste functionality for object container
    """
    
    def CanCopy(self):
        """
        """
        return not hasattr(self, "disableCopy") or not self.disableCopy


    def CanPaste(self):
        """
        """
        return not hasattr(self, "disablePaste") or not self.disablePaste


    def Paste(self, ids, pos, user):
        """
        Paste the copied object with id to this object
        """
        root = self.GetRoot()
        new = []
        msgs = []
        result = True
        for id in ids:
            id = int(id)
            if self.GetID() == id:
                continue
            obj = root.LookupObj(id, preload="skip")
            if not obj:
                msgs.append(_(u"Object not found"))
                result = False
                continue
            newobj = self.Duplicate(obj, user) 
            if not newobj:
                raise TypeError, "Duplicate failed"
            if ISort.providedBy(self):
                self.InsertAfter(newobj.id, pos, user=user)
            new.append(newobj)
        if not self.app.configuration.autocommit:
            for o in new:
                o.Commit(user)
        if result:
            msgs.append(_(u"OK. Copied and pasted."))
        return result, msgs
    

    def Move(self, ids, pos, user):
        """
        Move the object with id to this object
        
        Events
        
        - beforeAdd(data=obj.meta, type=type)
        - afterDelete(id=obj.id)
        - moved()
        - afterAdd(obj=obj)
        """
        root = self.root()
        oldParent=None

        moved = []
        msgs = []
        result = True
        for id in ids:
            id = int(id)
            if self.GetID() == id:
                continue
            obj = root.LookupObj(id, preload="skip")
            if not obj:
                msgs.append(_(u"Object not found"))
                result = False
                continue

            type=obj.GetTypeID()
            # allow subobject
            if not self.IsTypeAllowed(type, user):
                raise TypeError, "Object cannot be added here"

            self.Signal("beforeAdd", data=obj.meta, type=type)
            if not oldParent or oldParent.id != obj.GetParent().id:
                oldParent = obj.GetParent()
            obj.__parent__ = self
            obj.meta["pool_unitref"] = self.GetID()
            oldParent.Signal("afterDelete", id=obj.id)
            obj.Signal("moved")
            #obj.Close()
            moved.append(obj)

        for o in moved:
            o.Commit(user)
            if ISort.providedBy(self):
                self.InsertAfter(o.id, pos, user=user)
            self.Signal("afterAdd", obj=o)
        if result:
            msgs.append(_(u"OK. Cut and pasted."))
        return result, msgs
    


views = [
    # cut, copy
    ViewConf(name = "cut",  attr = "cut",  context = IContainer, permission="edit"),
    ViewConf(name = "copy", attr = "copy", context = IContainer, permission="edit"),    
    # paste
    ViewConf(name = "paste", attr = "paste", context = IContainer, permission = "add")
]


class CopyView:
    """
    View functions for cut, copy and paste
    """
    CopyInfoKey = "CCP__"

    def cut(self):
        """
        """
        ids = self.GetFormValue(u"ids")
        if not ids:
            ids = [self.context.id]
        cp = self.SetCopyInfo(u"cut", ids, self.context)
        url = self.GetFormValue(u"url")
        if not url:
            url = self.PageUrl(self.context)
        msgs = _(u"OK. Cut.")
        ok = True
        return self.Redirect(url, [msgs])


    def copy(self):
        """
        """
        ids = self.GetFormValue(u"ids")
        if not ids:
            ids = [self.context.id]
        cp = self.SetCopyInfo(u"copy", ids, self.context)
        url = self.GetFormValue(u"url")
        if not url:
            url = self.PageUrl(self.context)
        msgs = _(u"OK. Copied.")
        return self.Redirect(url, [msgs])


    def paste(self):
        """
        """
        deleteClipboard=1
        url = self.GetFormValue(u"url")
        if not url:
            url = self.PageUrl(self.context)
        action, ids = self.GetCopyInfo()
        if not action or not ids:
            msgs = []
            return self.Redirect(url, msgs)

        pepos = self.GetFormValue(u"pepos",0)
        result = False
        msgs = [_(u"Method unknown")]
        if action == u"cut":
            result, msgs = self.context.Move(ids, pepos, user=self.User())
            if result and deleteClipboard:
                cp = self.DeleteCopyInfo()
        elif action == u"copy":
            result, msgs = self.context.Paste(ids, pepos, user=self.User())
        return self.Redirect(url, msgs)
    
    
    def SetCopyInfo(self, action, ids, context):
        """
        store in session or cookie
        """
        if not ids:
            return u""
        if isinstance(ids, basestring):
            ids=ConvertToNumberList(ids)
        cp = ConvertListToStr([action]+ids).replace(u" ",u"")
        self.request.session[self.CopyInfoKey] = cp
        return cp


    def GetCopyInfo(self):
        """
        get from session or cookie
        """
        cp = self.request.session.get(self.CopyInfoKey,u"")
        if isinstance(cp, basestring):
            cp = cp.split(u",")
        if not cp or len(cp)<2:
            return u"", []
        return cp[0], cp[1:]

    
    def ClipboardEmpty(self):
        """
        check if clipboard is empty
        """
        cp = self.request.session.get(self.CopyInfoKey,u"")
        return cp==u""
    

    def DeleteCopyInfo(self):    
        """
        reset copy info
        """
        self.request.session[self.CopyInfoKey] = u""

    


#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = ""

import re, htmlentitydefs
import iso8601
import os, tempfile, json
from datetime import datetime
from mimetypes import guess_type, guess_extension
from operator import itemgetter, attrgetter
from types import StringType

from path import DvPath


def ConvertHTMLToText(html, url="", removeReST=True):
    # requires html2text module
    try:
        import html2text
    except ImportError:
        return html
    h = html2text.HTML2Text()
    h.ignore_links = True
    if removeReST:
        h.ignore_emphasis = True
        text = h.handle(html)
        # replace markdown #
        text = text.replace(u"# ","")
        return text
    return h.handle(html)


def ConvertTextToHTML(html):
    # replaces lines breaks with br's
    return html.replace(u"\n", u"\n<br/>")


def ConvertToDateTime(date):
    if isinstance(date, datetime):
        return date
    elif isinstance(date, float):
        return datetime.fromtimestamp(date)
    elif not date:
        return None
    try:
        return iso8601.parse_date(date)
    except (iso8601.ParseError, TypeError), e:
        pass
    # try other string format versions
    try: # 2011-12-23
        return datetime.strptime(date, "%Y-%m-%d")
    except ValueError:
        pass
    try: # 2011/12/23
        return datetime.strptime(date, "%Y/%m/%d")
    except ValueError:
        pass
    try: # 2011/12/23 13:22
        return datetime.strptime(date, "%Y/%m/%d %H:%M")
    except ValueError:
        pass
    try: # 2011/12/23 13:22:11
        return datetime.strptime(date, "%Y/%m/%d %H:%M:%S")
    except ValueError:
        pass



def XssEscape(html, permitted_tags=None, requires_no_close=None, allowed_attributes=None):
    """
    source: http://code.activestate.com/recipes/496942-cross-site-scripting-xss-defense/
    
    A list of the only tags allowed.  Be careful adding to this.  Adding
    "script," for example, would not be smart.  'img' is out by default 
    because of the danger of IMG embedded commands, and/or web bugs.
    permitted_tags = ['a', 'b', 'blockquote', 'br', 'i', 
                      'li', 'ol', 'ul', 'p', 'cite', 'strong', 'em', 'pre', 'h3', 'h4']

    A list of tags that require no closing tag.
    requires_no_close = ['img', 'br']

    A dictionary showing the only attributes allowed for particular tags.
    If a tag is not listed here, it is allowed no attributes.  Adding
    "on" tags, like "onhover," would not be smart.  Also be very careful
    of "background" and "style."
    allowed_attributes = \
            {'a':['href','title'],
             'img':['src','alt'],
             'blockquote':['type']}
    """
    import xssescape
    xss = xssescape.XssCleaner()
    if permitted_tags:
        xss.permitted_tags = permitted_tags
    if requires_no_close:
        xss.requires_no_close = requires_no_close
    if allowed_attributes:
        xss.allowed_attributes = allowed_attributes
    return xss.strip(html)


def CutText(text, textlen, cutchars=" ;:,.\r\n", postfix=u" ..."):
    """
    For text preview. cut the text at the last found char in cutchars.
    """
    if len(text)<textlen:
        return text
    pos = len(text)-1
    for c in cutchars:
        p = text.find(c, textlen)
        if p!=-1 and p<pos:
            pos=p
    return text[:pos] + postfix


def ConvertHexToBin(m):
    return u''.join(map(lambda x: chr(16*int(u'0x%s'%m[x*2],0)+int(u'0x%s'%m[x*2+1],0)),range(len(m)/2)))

def ConvertBinToHex(m):
    return u''.join(map(lambda x: hex(ord(x)/16)[-1]+hex(ord(x)%16)[-1],m))


def FormatBytesForDisplay(size):
    """Return the size of a file or directory formatted for display."""
    if size in (None,-1,0):
        return u""
    if size == 1:
        return u"1 byte"
    for factor, suffix in ((1<<30L, u"GB"),(1<<20L, u"MB"),(1<<10L, u"kB"),(1, u"bytes")):
        if size >= factor:
            break
    return u"%d %s" % (size / factor, suffix)

def FmtSeconds(seconds):
    # Format seconds for display
    #[$] seconds: seconds to convert
    if seconds is None: return '-' * 5
    if seconds == -1: return '-'

    minutesSingular = u'%d minute '
    minutesPlural = u'%d minutes '
    hoursSingular = u'%d hour '
    hoursPlural = u'%d hours '

    k = 60
    if (seconds > k):
        t2 = seconds / k
        if (t2 > k):
            t3 = t2 / k
            s = u""
            if t3 == 1:
                s += hoursSingular % t3
            else:
                s += hoursPlural % t3
            m = t2%k
            if m==1:
                s += minutesSingular % m
            else:
                s += minutesPlural % m
            return s
        else:
            if t2 == 1:
                return minutesSingular % t2
            return minutesPlural % t2
    else:
        return u'%d seconds' % seconds


def SortConfigurationList(values, sort, ascending=True):
    """
    Sorts the dictionary list `values` by attribute or key `sort`. This works for definitions.Conf() objects 
    and simple dictionaries. 
    Results can be ordered ascending or descending.
    """
    if not sort:
        return values
    try:
        values = sorted(values, key=attrgetter(sort))   
    except AttributeError:
        values = sorted(values, key=itemgetter(sort))   
    if not ascending:
        values.reverse()
    return values


def GetMimeTypeExtension(extension):
    if extension.find(".") != -1:
        extension = DvPath(extension).GetExtension()
    # custom and uncommeon
    if extension == u"fla":          return u"application/flash"
    # standard
    elif extension == u"html":       return u"text/html"
    elif extension == u"txt":        return u"text/plain"
    elif extension == u"dtml":       return u"text/html"
    elif extension == u"jpg":        return u"image/jpeg"
    elif extension == u"gif":        return u"image/gif"
    elif extension == u"png":        return u"image/png"
    elif extension == u"jpeg":       return u"image/jpeg"
    elif extension == u"psd":        return u"image/psd"
    elif extension == u"pdf":        return u"application/pdf"
    elif extension == u"swf":        return u"application/x-shockwave-flash"
    elif extension == u"flv":        return u"application/x-shockwave-flash"
    elif extension == u"dcr":        return u"application/x-director"
    elif extension == u"doc":        return u"application/msword"
    elif extension == u"xls":        return u"application/vnd.ms-excel"
    elif extension == u"ppt":        return u"application/vnd.ms-powerpoint"
    elif extension == u"mpp":        return u"application/vnd.ms-project"
    elif extension == u"gz":         return u"application/gz"
    elif extension == u"rtf":        return u"text/rtf"
    elif extension == u"dat":        return u"application/octet-stream"
    elif extension == u"flv":        return u"video/flv"
    elif extension == u"ogv":        return u"video/ogg"
    elif extension == u"webm":       return u"video/webm"
    elif extension == u"mp4":        return u"video/mp4"
    elif extension == u"mp3":        return u"audio/mp3"
    elif extension == u"ogg":        return u"audio/ogg"
    e = guess_type(u"x." + extension)
    if e[0]:                return e[0]
    return u""

def GetExtensionMimeType(mime):
    # custom and uncommeon
    if mime == u"application/flash":  return u"fla"
    # standard
    elif mime == u"text/html":        return u"html"
    elif mime == u"text/plain":       return u"txt"
    elif mime == u"text/html":        return u"dtml"
    elif mime == u"image/jpeg":       return u"jpg"
    elif mime == u"image/gif":        return u"gif"
    elif mime == u"image/png":        return u"png"
    elif mime == u"image/jpeg":       return u"jpeg"
    elif mime == u"image/psd":        return u"psd"
    elif mime == u"application/pdf":                  return u"pdf"
    elif mime == u"application/x-shockwave-flash":    return u"swf"
    elif mime == u"application/x-director":           return u"dcr"
    elif mime == u"application/msword":               return u"doc"
    elif mime == u"application/vnd.ms-excel":         return u"xls"
    elif mime == u"application/vnd.ms-word":          return u"doc"
    elif mime == u"application/vnd.ms-powerpoint":    return u"ppt"
    elif mime == u"application/vnd.ms-project":       return u"mpp"
    elif mime == u"text/rtf":                         return u"rtf"
    elif mime == u"application/octet-stream":         return u"dat"
    elif mime == u"video/flv":        return u"flv"
    elif mime == u"video/ogg":        return u"ogv"
    elif mime == u"video/webm":       return u"webm"
    elif mime == u"video/mp4":        return u"mp4"
    elif mime == u"audio/mp3":        return u"mp3"
    elif mime == u"audio/ogg":        return u"ogg"

    m = guess_extension(mime)
    if m:                            return m[1:]
    return u""



def TidyHtml(data, options=None):
    """
    clean up html by calling tidy
    """
    if not options:
        options = { 'output-xhtml'       : '1',
                    'add-xml-decl'       : '1',
                    'indent'             : 'auto',
                    'tidy-mark'          : '0',
                    'char-encoding'      : 'utf8',
                    'clean'              : '0',
                    'drop-font-tags'     : '1',
                    'enclose-block-text' : '0',
                    'logical-emphasis'   : '1',
                    'word-2000'          : '1',
                    'wrap'               : '65',
                    'write-back'         : '0'}

    cmds = ""
    for k in options.keys():
        cmds += " --"+k+" "+options[k]

    # write file
    file, filename = tempfile.mkstemp(suffix='.html', prefix='tidy')
    os.write(file, data)
    os.close(file)

    # call tidy
    out = filename + "_tidy"
    cmd = """%(cmds)s -o %(out)s %(file)s""" %{"cmds": cmds, "out": out, "file": filename}

    s = os.popen("tidy " + cmd, "r")
    r = ""
    while 1:
        d = s.readline()
        if not d or d == ".\n" or d == ".\r":
            break
        r += d

    # delete file
    try:
        os.remove(filename)
    except:
        pass

    # return output
    try:
        file = open(out, "r+b")
        data2 = file.read()
        file.close()
        os.remove(out)
    except:
        return data
    return data2


def ReplaceHTMLEntities(text, codepage = None):
    ##
    # Removes HTML or XML character references and entities from a text string.
    #
    # @param text The HTML (or XML) source text.
    # @return The plain text, as a Unicode string, if necessary.
    def _fixup(m):
        text = m.group(0)
        if text[:2] == u"&#":
            # character reference
            try:
                if text[:3] == u"&#x":
                    return unichr(int(text[3:-1], 16))
                else:
                    return unichr(int(text[2:-1]))
            except ValueError:
                pass
        else:
            # named entity
            try:
                text = unichr(htmlentitydefs.name2codepoint[text[1:-1]])
            except KeyError:
                pass
        return text # leave as is

    if codepage:
        text = unicode(text, codepage)
    result = re.sub(u"&#?\w+;", _fixup, text)
    if codepage:
        result = result.encode(codepage)
    return result


def LoadFromFile(path):
    file = open(path)
    if not file:
        return None
    value = file.read()
    file.close()
    return value


def WriteToFile(path, value, append = False):
    if append:
        file = open(path, "ab+")
        if not file:
            return False
    else:
        file = open(path, "wb")
        if not file:
            return False
    file.write(value)
    file.close()
    return True


# failsafe type conversion ---------------------------------------------------------------------------------------

def ConvertToStr(data, sep=u"\n"):
    if isinstance(data, (list, tuple)):
        return ConvertListToStr(data)
    elif isinstance(data, dict):
        v = []
        for key, value in data.items():
            v.append(u"%s: %s"%(unicode(key), ConvertToStr(value, sep)))
        return sep.join(v)
    return unicode(data)

def ConvertToBool(data, raiseExcp = False):
    try:
        return int(data)
    except:
        if isinstance(data, basestring):
            if data.lower() in (u"true", u"yes", u"checked", u"ja", u"qui", u"si"):
                return True
            if data.lower() in (u"false", u"no", u"nein", u"non"):
                return False
        if raiseExcp:
            raise
        return False

def ConvertToInt(data, raiseExcp = False):
    try:
        return int(data)
    except:
        if ConvertToBool(data, raiseExcp):
            return 1
        else:
            return 0
        if raiseExcp:
            raise 
        return 0

def ConvertToFloat(data, raiseExcp = False):
    try:
        return float(data)
    except:
        if ConvertToBool(data, raiseExcp):
            return 1.0
        else:
            return 0.0
        if raiseExcp:
            raise 
        return 0.0

def ConvertToLong(data, raiseExcp = False):
    try:
        return long(data)
    except:
        if ConvertToBool(data, raiseExcp):
            return 1L
        else:
            return 0L
        if raiseExcp:
            raise 
        return 0L

def ConvertToList(data, raiseExcp = False):
    """
    converts a string to list.
    the list items can be seperated by "," or "\n"
    """
    try:
        if not data:
            return []
        if isinstance(data, (list,tuple)):
            return data
        data = data.strip()
        if not len(data):
            return []
        if data[0] in (u"[", u"("):
            data = data[1:-1]

        sep = u","
        if data.find(u"\n") != -1:
            sep = u"\n"

        result = []
        l = data.split(sep)
        for value in l:
            value = value.strip()
            if not len(value):
                continue
            if value[0] in (u"'", u"\""):
                value = value[1:-1]
            result.append(value)
        return result
    except:
        if raiseExcp:
            raise 
        return []

def ConvertToNumberList(data, raiseExcp = False):
    try:
        l = ConvertToList(data, raiseExcp)
        for i in range(len(l)):
            l[i] = ConvertToLong(l[i], raiseExcp)
        return l
    except:
        if raiseExcp:
            raise 
        return []


def ConvertListToStr(values, sep = u", ", textMarker = u"", keepType = False):
    """
    converts a python list to string. .
    the list items are seperated by ",". list items are converted to string
    """
    if not values:
        return u""
    if isinstance(values, basestring):
        return u"%s%s%s" % (textMarker, values, textMarker)
    s = []
    for v in values:
        if textMarker != u"":
            tm = textMarker
            if keepType:
                if isinstance(v, (int, long, float)):
                    tm = u""
            s.append(u"%s%s%s" % (textMarker, unicode(v), textMarker))
        else:
            s.append(unicode(v))
    return sep.join(s)


def ConvertDictToStr(values, sep = u"\n"):
    """
    converts a python dictionary to key list string.
    the list items are seperated by ",". list items are converted to string
    
    ::
        key1: value1
        key2: value2
        
    result string
    """
    s = [u"%s: %s" % (key, ConvertToStr(value, sep)) for key, value in values.items()]
    return sep.join(s)


def StrAscii(s):
    return filter(lambda x: x in string.ascii_lowercase+"0123456789_.", s.lower())


# Logging --------------------------------------------------------------------------


def DUMP(data, path = "dump.txt", addTime=False):
    if addTime:
        date = datetime.now()
        aS = "\r\n\r\n" + date.strftime("%Y-%m-%d %H:%M:%S") + "\r\n" + str(data)
    else:
        aS = "\r\n\r\n" + str(data)
    WriteToFile(path, True)

def STACKF(t=0, label = "", limit = 15, path = "_stackf.txt", name=""):
    import time
    import traceback
    n = time.time() - t
    date = datetime.now()
    h = "%s: %f (%s)" % (date.strftime("%Y-%m-%d %H:%M:%S"),n,name)
    if limit<2:
        DUMP("%s\r\n%s\r\n" % (h, label), path)
        return
    import StringIO
    s = StringIO.StringIO()
    traceback.print_stack(limit=limit, file=s)
    DUMP("%s\r\n%s\r\n%s" % (h, label, s.getvalue()), path)

def LOG(data = "", path="_log.txt", t=0):
    import time
    n = time.time() - t
    try:
        fd = open(path, "ab+")
        fd.write("%f %s\r\n" % (n, data))
        fd.close()
    except:
        try:
            fd.close()
        except:
            pass



class logdata:
    _data = ""
    _maxsize = 1024 * 512
    _path = ""
    
    def __del__(self):
        self.write("", 1)

    def write(self, data="", flush=0):
        import thread
        if flush or len(self._data)>self._maxsize:
            try:
                lock = thread.allocate_lock()
                lock.acquire(1)
                fd = open(self._path, "ab+")
                fd.write(self._data + data)
                self._data = ""
                fd.close()
                if lock.locked():
                    lock.release()
                return True
            except:
                try:
                    fd.close()
                except:
                    pass
                try:
                    if lock.locked():
                        lock.release()
                except:
                    pass
                return False
        else:
            try:
                lock = thread.allocate_lock()
                lock.acquire(1)
                self._data += data
                if lock.locked():
                    lock.release()
                return True
            except:
                return False

# global log
_log = None

def LOGHTTP(client="-", user="-", t=0, action="GET", url="-", state="200", referrer="-", agent="-", path="http.log", logObj=None):
    import time
    if t != 0:
        t = time.localtime(t)
    else:
        t = time.localtime()
    t = time.strftime(u"[%d/%m/%Y:%H:%M:%S", t)
    ms = ("%f"%(time.time())).split(".")[-1]
    t += ".%s]"%(ms)
    referrer = referrer.replace('"', '')
    agent = agent.replace('"', '')
    log = u"""%s - "%s" %s "%s %s" %s - "%s" "%s"\r\n""" % (client, user, t, action, url, str(state), referrer, agent)
    if not logObj:
        logObj = _log
    if not logObj:
        logObj = logdata()
        logObj._path = path
    return logObj.write(log)
    




#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = ""

import types


from nive.tools import Tool
from nive.definitions import ToolConf, FieldConf


configuration = ToolConf()
configuration.id = "exampletool"
configuration.context = "nive.components.tools.example.tool"
configuration.name = u"Empty tool for tests"
configuration.description = ""
configuration.apply = None  #(IApplication,)
configuration.data = [
    FieldConf(**{"id": "parameter1", "datatype": "bool",     "required": 0,     "readonly": 0, "default": 0, "name": u"Show 1",    "description": u"Display 1"}),
    FieldConf(**{"id": "parameter2", "datatype": "string",     "required": 0,     "readonly": 0, "default": 0, "name": u"Show 2",    "description": u"Display 2"})
]
configuration.mimetype = "text/html"



class tool(Tool):

    def _Run(self, **values):
        result = u"<h1>OK</h1>"
        return result





import time
import unittest

from nive.definitions import OperationalError
from nive.application import *
from nive.definitions import *
from nive.helper import *
from nive.events import Events
from nive.portal import Portal

from nive.tests import db_app

mApp = AppConf(id="app", groups=[GroupConf(id="g1",name="G1")], categories=[CategoryConf(id="c1",name="C1")])
dbConfiguration = DatabaseConf(dbName="test")

mObject = ObjectConf(id="object", dbparam="object", name="Object",
                     data=(FieldConf(id="a1",datatype="string",name="A1"),FieldConf(id="a2",datatype="number",name="A2"),))

mRoot = RootConf(id="root")
mTool = ToolConf(id="tool", context="nive.components.tools.example")
mViewm = ViewModuleConf(id="vm")
mView = ViewConf(id="v",context=mApp,view=mApp)
mMod = ModuleConf(id="mod", context=mApp)



mApp2 = AppConf(id="app2", 
                context="nive.tests.test_nive.testapp",
                modules=[mObject,mRoot,mTool,mViewm,mView,mMod], 
                groups=[GroupConf(id="g1",name="G1")], 
                categories=[CategoryConf(id="c1",name="C1")])
dbConfiguration2 = DatabaseConf(dbName="test")



class testapp(Application, Registration, Configuration, AppFactory, Events):
    """
    """
    
def app():
    app = testapp()
    app.Register(mApp2)
    app.Register(dbConfiguration2)
    app.Startup(None)
    portal = Portal()
    portal.Register(app, "nive")
    return app



class modTest(unittest.TestCase):
    
    def setUp(self):
        self.app = testapp()

    def tearDown(self):
        pass


    def test_Register(self):
        self.app.Register(mApp)
        self.app.Register(dbConfiguration)
        self.app.Register(mObject)
        self.app.Register(mRoot)
        self.app.Register(mTool)
        self.app.Register(mViewm)
        self.app.Register(mView)
        self.app.Register(mMod)
        self.app.Register(ModuleConf(id="aaa"), provided=IObject, name="testttttt")
        self.app.Startup(None)
        self.assert_(self.app.db)

    def test_include2(self):
        self.app.Register(mApp2)
        self.app.Register(dbConfiguration2)
        self.app.Startup(None)
        self.assert_(self.app.db)

    def test_includefailure(self):
        self.app.Register(mApp2)
        #no database: self.app.Register(dbConfiguration2)
        self.assertRaises(OperationalError, self.app.Startup, None)


class appTest(unittest.TestCase):
    
    def setUp(self):
        self.app = testapp()
        self.app.Register(mApp2)
        self.app.Register(dbConfiguration2)
        self.portal = Portal()
        self.portal.Register(self.app, "nive")
        self.app.Startup(None)

    def tearDown(self):
        pass


    def test_include2(self):
        s = self.app._structure.structure
        self.assert_("pool_meta" in s)
        self.assert_("object" in s)
        self.assert_(len(s["pool_meta"])>10)
        self.assert_(len(s["object"])==2)
        pass


    def test_fncs(self):
        self.assert_(self.app.GetVersion())
        self.assert_(self.app.CheckVersion())
        

    def test_objs(self):
        self.assert_(self.app.root(name=""))
        self.assert_(self.app.root(name="root"))
        self.assert_(self.app.root(name="aaaaaaaa")==None)
        #self.assert_(self.app.portal)
        self.assert_(self.app.__getitem__("root"))
        self.assert_(self.app.GetRoot(name="root"))
        self.assert_(len(self.app.GetRoots())==1)
        self.assert_(self.app.GetApp())
        #self.assert_(self.app.portal)
        self.assert_(self.app.GetTool("nive.components.tools.example"))
        #!!! self.app.GetWorkflow(wfProcID)

    def test_confs(self):
        self.assert_(self.app.GetRootConf(name=""))
        self.assert_(self.app.GetRootConf(name="root"))
        self.assert_(len(self.app.GetAllRootConfs()))
        self.assert_(len(self.app.GetRootIds())==1)
        self.assert_(self.app.GetDefaultRootName()=="root")

    def test_confs2(self):
        self.assert_(self.app.GetObjectConf("object", skipRoot=False))
        self.assert_(self.app.GetObjectConf("root", skipRoot=True)==None)
        self.assert_(self.app.GetObjectConf("oooooh", skipRoot=False)==None)
        self.assert_(len(self.app.GetAllObjectConfs(visibleOnly = False))==1)
        self.assert_(self.app.GetTypeName("object")=="Object")

    def test_confs3(self):
        self.assert_(self.app.GetToolConf("tool"))
        self.assert_(len(self.app.GetAllToolConfs()))
        
    def test_confs4(self):
        self.assert_(self.app.GetCategory(categoryID = "c1"))
        self.assert_(len(self.app.GetAllCategories(sort=u"name", visibleOnly=False))==1)
        self.assert_(self.app.GetCategoryName("c1")=="C1")
        self.assert_(self.app.GetGroups(sort=u"name", visibleOnly=False))
        self.assert_(self.app.GetGroupName("g1")=="G1")

    def test_flds(self):
        self.assert_(self.app.GetFld("title", typeID = None))
        self.assert_(self.app.GetFld("aaaaa", typeID = None)==None)
        self.assert_(self.app.GetFld("pool_stag", typeID = "object"))
        self.assert_(self.app.GetFld("a1", typeID = "object"))
        self.assert_(self.app.GetFld("a1", typeID = None)==None)
        self.assert_(self.app.GetFld("a2", typeID = "object"))
        self.assert_(self.app.GetFld("a2", typeID = "ooooo")==None)
        self.assert_(self.app.GetFld("ooooo", typeID = "object")==None)
        
        self.assert_(self.app.GetFldName("a2", typeID = "object")=="A2")
        self.assert_(self.app.GetObjectFld("a1", "object"))
        self.assert_(len(self.app.GetAllObjectFlds("object"))==2)
        self.assert_(self.app.GetMetaFld("title"))
        self.assert_(len(self.app.GetAllMetaFlds(ignoreSystem = True)))
        self.assert_(len(self.app.GetAllMetaFlds(ignoreSystem = False)))
        self.assert_(self.app.GetMetaFldName("title")=="Title")
        
    def test_flds2(self):    
        self.app.LoadStructure(forceReload = False)
        self.assert_(self.app._structure)
        self.app.LoadStructure(forceReload = True)
        self.assert_(self.app._structure)

    def test_flds4(self):    
        self.assert_(self.app._GetDataPoolObj())
        self.assert_(self.app._GetRootObj("root"))
        self.app._CloseRootObj(name="root")
        self.assert_(self.app._GetRootObj("root"))
        self.assert_(self.app._GetToolObj("nive.components.tools.example", None))



class appTest_db:
    
    def setUp(self):
        self.app = db_app.app_db()
        
    def tearDown(self):
        pass


    def test_database(self):
        r = self.app.root()
        o = db_app.createObj1(r)
        self.assert_(o)
        id = o.id
        del o
        self.assert_(self.app.obj(id, rootname = ""))
        self.assert_(self.app.obj(id, rootname = "root"))
        self.assert_(self.app.db)
        self.assert_(self.app.GetDB())
        self.assert_(self.app.db.connection.VerifyConnection())
        self.assert_(len(self.app.Query("select id from pool_meta", values = [])))
        ph = self.app.db.placeholder
        self.assert_(len(self.app.Query("select id from pool_meta where pool_type="+ph, values = ["type1"])))
        o=self.app.obj(id, rootname = "")
        self.assert_(o)
        self.app.Register("nive.components.tools.example")
        self.assert_(self.app.GetTool("exampletool"))
        user = User(u"test")
        self.app.Close()
        self.assertFalse(self.app.db.usedconnection.IsConnected())
        # should reopen the connection
        self.assert_(self.app.LookupObj(id))
        self.app.root().Delete(id, user=user)


class appTest_db_(appTest_db, unittest.TestCase):
    """
    """


if __name__ == '__main__':
    unittest.main()



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
Image processing extension
--------------------------
Converts images based on profiles and stores the converted images as different
files in object.
Profiles are configured as ``object.configuration.imageProfiles`` with the following attributes: ::

    source = "imagefull": field id of the source image 
    dest = "image": field id of the destination image
    format = "jpg": image format
    quality = "80": quality depending on format
    width = 400: width of new image
    height = 0: height of new image
    extension = "jpg" : file name extension
    condition = None : callback function which takes the image object as parameter and returns whether
                       the profile is valid for the image object or not 

Examples ::

    ProfileImage = Conf(source="imagefull", dest="image", format="JPEG", 
                        quality="80", width=400, height=0, extension="jpg")
    ProfileIcon =  Conf(source="imagefull", dest="icon",  format="JPEG", 
                        quality="70", width=100, height=0, extension="jpg")
    
If either width or height is 0 the new image is scaled proportionally.
See PIL documentation for possible format and quality values.
 
"""

from nive.definitions import Conf
from nive.utils.path import DvPath
from nive.utils.dataPool2.files import File
from nive.helper import ResolveName
from nive.i18n import _

import sys
from time import time

try:     from cStringIO import StringIO
except:  from StringIO import StringIO

try:     from win32pipe import popen
except:  from os import popen

PILloaded = 1
try:     from PIL import Image
except:  PILloaded=0


class ImageProcessor(object):

    def GetImgSize(self, image):
        """
        Returns the image size (width, height) in pixel
        """
        i = self.files.get(image)
        if not i:
            return 0,0
        try:
            im = Image.open(i.path)
            return im.size
        except:
            return 0,0        
        

    def Process(self, images=None, profiles=None):
        """
        Process images and create versions from profiles. ::
        
            images = list of image field ids to process. if none all images based on 
                     selected profiles are processed.
            profiles = list of profiles to process. if none all profiles are 
                       processed
            returns result, messages
        
        Example: If `Process(images=['highres'],profiles=None)` all versions with
        `source=highres` are updated.

        Events: 
        updateImage(profile)

        """
        if not PILloaded:
            return False, [_(u"Python image library (PIL) not installed.")]
        convert = []
        if not images:
            for p in self.configuration.imageProfiles:
                if not self._CheckCondition(p):
                    continue
                convert.append(p)
        else:
            for p in self.configuration.imageProfiles:
                if not self._CheckCondition(p):
                    continue
                if p.source in images:
                    convert.append(p)
        if not convert:
            return True
        msgs = []
        result = True
        for profile in convert:
            r, m = self._Convert(profile)
            msgs += m
            if not r:
                result = False
            else:
                self.Signal("updateImage", profile=profile)
        return result, msgs
        
        
    def _CheckCondition(self, profile):
        if not hasattr(profile, "condition"):
            return True
        c = ResolveName(profile.condition)
        return c(self)
      
        
    def _Convert(self, profile):
        source = self.files.get(profile.source)
        if not source or not source.tempfile:
            # convert only if tempfile
            return False, ()
        if not source:
            return False, [_(u"Image not found: ") + profile.source]
        p = DvPath()
        p.SetUniqueTempFileName()
        p.SetExtension(profile.extension)
        destPath = str(p)
        
        try:
            source.file.seek(0)
        except:
            pass
        try:
            iObj = Image.open(source)
        except IOError:
            # no file to be converted
            return False, ()
        iObj = iObj.convert("RGB")
        
        # resize
        size = [profile.width, profile.height]
        if size[0] != 0 or size[1] != 0:
            if size[0] == 0:    
                size[0] = size[1]
            elif size[1] == 0:    
                size[1] = size[0]
            x, y = iObj.size
            if x > size[0]: y = y * size[0] / x; x = size[0]
            if y > size[1]: x = x * size[1] / y; y = size[1]
            size = x, y
        
        iObj = iObj.resize(size, Image.ANTIALIAS)
        iObj.save(destPath, profile.format)
        try:
            source.file.seek(0)
        except:
            pass
        
        # file meta data
        imgFile = open(destPath)
        filename = DvPath(profile.dest+"_"+source.filename)
        filename.SetExtension(profile.extension)
        file = File(filekey=profile.dest, 
                    filename=str(filename), 
                    file=imgFile,
                    size=p.GetSize(), 
                    path=destPath, 
                    extension=profile.extension,  
                    tempfile=True)
        self.files.set(profile.dest, file)
        return True, []
        

    def Init(self):
        if PILloaded:
            self.ListenEvent("commit", "ProcessImages")

    def ProcessImages(self):
        images = []
        keys = self.files.keys()
        for p in self.configuration.imageProfiles:
            if p.source in images:
                continue
            if not p.source in keys:
                continue
            f = self.files.get(p.source)
            if not f or not f.tempfile:
                continue
            images.append(p.source)
        self.Process(images=images)

      


# -*- coding: utf-8 -*-

import time
import unittest

from nive.definitions import FieldConf
from nive.portal import Portal
from nive.i18n import _

from nive.userdb.tests.db_app import *
from nive.userdb.useradmin import view
from nive.userdb.useradmin import adminroot

from pyramid.httpexceptions import HTTPFound
from pyramid import testing 
from pyramid.renderers import render



class tViews(unittest.TestCase):

    def setUp(self):
        request = testing.DummyRequest()
        request._LOCALE_ = "en"
        self.request = request
        self.config = testing.setUp(request=request)
        self.app = app()
        self.portal = Portal()
        self.portal.Register(self.app, "nive")
        self.app.Register("nive.cms.cmsview.view")
        self.app.Register(adminroot.configuration)
        self.app.Startup(self.config)
        self.root = self.app.root("usermanagement")
        self.request.context = self.root
    

    def tearDown(self):
        self.app.Close()
        testing.tearDown()


    def test_views(self):
        v = view.UsermanagementView(context=self.root, request=self.request)
        self.assert_(v.GetAdminWidgets())
        self.assert_(v.add())
        self.assert_(v.edit())
        self.assert_(v.delete())
        v.view()

    def test_templates(self):
        user = User(u"test")
        v = view.UsermanagementView(context=self.root, request=self.request)
        vrender = {"context":self.root, "view":v, "request": self.request}
        
        values = v.add()
        values.update(vrender)
        render("nive.userdb.useradmin:add.pt", values)
        u=self.root.Create("user",{"name":"testuser", "password":"test", "email":"test@aaa.com", "groups":("group:admin",)},user=user)
        v.context = u
        values = v.edit()
        values.update(vrender)
        values["context"] = u
        render("nive.userdb.useradmin:edit.pt", values)
        v.context = self.root
        values = v.view()
        values.update(vrender)
        render("nive.userdb.useradmin:root.pt", values)
        values = v.delete()
        values.update(vrender)
        render("nive.userdb.useradmin:delete.pt", values)
        
        self.root.Delete(u.id, user=user)


    def test_add(self):
        user = self.root.GetUserByName("testuser")
        if user:
            self.root.DeleteUser("testuser")
        v = view.UsermanagementView(context=self.root, request=self.request)
        r = v.add()
        self.assert_(r["result"])
        if self.root.GetUserByName("testuser"):
            self.assert_(False, "User should not exist")
        self.request.POST = {"name":"testuser", "password":"", "email":"test@aaa.com", "groups":("group:admin",)}
        r = v.add()
        self.assert_(r["result"])
        if self.root.GetUserByName("testuser"):
            self.assert_(False, "User should not exist")
        
        self.request.POST["create$"] = "create"
        r = v.add()
        self.assertFalse(r["result"])
        if self.root.GetUserByName("testuser"):
            self.assert_(False, "User should not exist")
        self.request.POST["password"] = "password"
        self.request.POST["password-confirm"] = "password"
        
        self.assertRaises(HTTPFound, v.add)
        self.assert_(self.root.GetUserByName("testuser"))
        
        self.root.DeleteUser("testuser")


    def test_edit(self):
        user = self.root.GetUserByName("testuser")
        if user:
            self.root.DeleteUser("testuser")
        v = view.UsermanagementView(context=self.root, request=self.request)
        self.request.POST = {"name":"testuser", "email":"test@aaa.com", "groups":("group:admin",)}
        self.request.POST["password"] = "password"
        self.request.POST["password-confirm"] = "password"
        self.request.POST["create$"] = "create"
        self.assertRaises(HTTPFound, v.add)
        user = self.root.GetUserByName("testuser")
        if not user:
            self.assert_(False, "User should exist")
        
        v = view.UsermanagementView(context=user, request=self.request)
        self.request.POST = {}
        r = v.edit()
        self.assert_(r["result"])

        self.request.POST = {"name":"testuser", "email":"test", "groups":("group:admin",)}
        self.request.POST["edit$"] = "edit"
        r = v.edit()
        self.assertFalse(r["result"])
        user = self.root.GetUserByName("testuser")
        self.assert_(user.data.email != "test")
        
        self.request.POST = {"name":"testuser", "email":"test@bbb.com", "groups":("group:admin",)}
        self.request.POST["edit$"] = "edit"
        r = v.edit()
        self.assert_(r["result"])
        user = self.root.GetUserByName("testuser")
        self.assert_(user.data.email == "test@bbb.com")
        
        self.root.DeleteUser("testuser")


    def test_delete(self):
        v = view.UsermanagementView(context=self.root, request=self.request)
        user = self.root.GetUserByName("testuser")
        if not user:
            self.request.POST = {"name":"testuser", "email":"test@aaa.com", "groups":("group:admin",)}
            self.request.POST["password"] = "password"
            self.request.POST["password-confirm"] = "password"
            self.request.POST["create$"] = "create"
            self.assertRaises(HTTPFound, v.add)
        user = self.root.GetUserByName("testuser")
        if not user:
            self.assert_(False, "User should exist")
            
        r = v.delete()
        self.assert_(r["result"])
        self.assertFalse(r["confirm"])

        ids = (user.id,)
        self.request.POST = {"ids":ids}
        r = v.delete()
        self.assert_(r["result"])
        self.assertFalse(r["confirm"])
        self.assert_(len(r["users"])==1)

        self.request.POST = {"ids":ids, "confirm": 1}
        self.assertRaises(HTTPFound, v.delete)
        self.assertFalse(self.root.GetUserByName("testuser"))
        

if __name__ == '__main__':
    unittest.main()



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = """
User and security functions.
"""


from pyramid.security import Allow 
from pyramid.security import Deny
from pyramid.security import ALL_PERMISSIONS
from pyramid.security import Everyone, Authenticated
from pyramid.security import remember, forget, authenticated_userid
from pyramid import threadlocal 
from pyramid.interfaces import IAuthenticationPolicy, IAuthorizationPolicy

from nive.definitions import ModuleConf, Conf
from nive.definitions import Interface, implements


def GetUsers(app):
    """
    Loads all users from user database if available.
    """
    portal = app.portal
    try:
        userdb = portal.userdb
        return userdb.root().GetUsers()
    except:
        return []

    
class User(object):
    """
    A fake user object for testing.
    """
    def __str__(self):
        return self.name
    
    def __init__(self, name, id=0):
        self.name = name
        self.id = id
        self.groups = []
        
    def GetGroups(self, context=None):
        return self.groups
    



class Unauthorized(Exception):
    """
    failed login
    """

class UserFound(Exception):
    """
    Can be used in *getuser* listeners to break user lookup and
    pass a session user to LookupUser. The second argument is the session
    user 
    """
    def __init__(self, user):
        self.user = user



class IAdminUser(Interface):
    """
    Used for admin user instance hard coded in configration 
    """


class AdminUser(object):
    """
    Admin User object with groups and login possibility. 
    """
    implements(IAdminUser)
    
    def __init__(self, values, ident):
        self.id = 0
        self.data = Conf(**values)
        self.meta = Conf()
        self.identity = ident or str(self.id)
        self.groups = self.data.groups = (u"group:admin",)

    def __str__(self):
        return str(self.identity)

    def Authenticate(self, password):
        return password == self.data["password"]
    
    def Login(self):
        """ """

    def Logout(self):
        """ """

    def GetGroups(self, context=None):
        """ """
        return self.groups

    def InGroups(self, groups):
        """
        check if user has one of these groups
        """
        if isinstance(groups, basestring):
            return groups in self.groups
        for g in groups:
            if g in self.groups:
                return True
        return False
    
    def ReadableName(self):
        return self.data.name

    
def effective_principals(request=None):
    # uses pyramid authentication groupfinder callback to lookup principals
    # returns None if no auth policy is configured (e.g. in tests)
    registry = threadlocal.get_current_registry()
    request = request or threadlocal.get_current_request()
    authn_policy = registry.queryUtility(IAuthenticationPolicy)
    authz_policy = registry.queryUtility(IAuthorizationPolicy)
    if authn_policy and authz_policy:
        principals = authn_policy.effective_principals(request)
        return principals
    return None


        



import time

from nive.definitions import *
from nive.security import User

from nive.tests import db_app




level1 = 5
level2 = 30
level3_1 = 9
level3_2 = 10


def test1():
    print "1) Testing objects create / delete, 3 level, no files"
    t = time.time()
    a=db_app.app_db()
    print time.time() - t, "Creating db (contains", db_app.statdb(a), "Entries)"
    t2 = time.time()
    r=a.root()
    print time.time() - t2, "Creating root"
    t2 = time.time()

    c=r
    ids=[]
    n=0
    user = User(u"test")
    for i in range(0,level1):
        o=db_app.createObj1(c)
        n+=1
        ids.append(o.id)
        for i2 in range(0,level2):
            o2=db_app.createObj1(o)
            n+=1
            for i3 in range(0,level3_1):
                db_app.createObj1(o2)
                n+=1
            for i3 in range(0,level3_2):
                db_app.createObj2(o2)
                n+=1

    print time.time() - t2, "Creating", n, "objects"
    t2 = time.time()

    for id in ids:
        r.Delete(id, user=user)
    print time.time() - t2, "Deleting objects recursively"

    print "------------"
    print time.time() - t, "Total. Finished (contains", db_app.statdb(a), "Entries)"
    a.Close()
    print "--------------------------------------------------------"


def test2():
    print "2) Testing objects create / delete, 3 level, with files"
    t = time.time()
    a=db_app.app_db()
    print time.time() - t, "Creating db (contains", db_app.statdb(a), "Entries)"
    t2 = time.time()
    r=a.root()
    print time.time() - t2, "Creating root"
    t2 = time.time()

    c=r
    ids=[]
    n=0
    for i in range(0,level1):
        o=db_app.createObj1file(c)
        n+=1
        ids.append(o.id)
        for i2 in range(0,level2):
            o2=db_app.createObj1file(o)
            n+=1
            for i3 in range(0,level3_1):
                db_app.createObj1file(o2)
                n+=1
            for i3 in range(0,level3_2):
                db_app.createObj2file(o2)
                n+=1

    print time.time() - t2, "Creating", n, "objects"
    t2 = time.time()

    user = User(u"test")
    for id in ids:
        r.Delete(id, user=user)
    print time.time() - t2, "Deleting objects recursively"

    print "------------"
    print time.time() - t, "Total Finished (contains", db_app.statdb(a), "Entries)"
    a.Close()
    print "--------------------------------------------------------"



def test3():
    print "3) Testing objects create / load / load batch / delete, 3 level, no files"
    t = time.time()
    a=db_app.app_db()
    print time.time() - t, "Creating db (contains", db_app.statdb(a), "Entries)"
    t2 = time.time()
    r=a.root()
    print time.time() - t2, "Creating root"
    t2 = time.time()

    c=r
    ids=[]
    n=0
    for i in range(0,level1):
        o=db_app.createObj1(c)
        n+=1
        ids.append(o.id)
        for i2 in range(0,level2):
            o2=db_app.createObj1(o)
            n+=1
            for i3 in range(0,level3_1):
                db_app.createObj1(o2)
                n+=1
            for i3 in range(0,level3_2):
                db_app.createObj2(o2)
                n+=1

    print time.time() - t2, "Creating", n, "objects"
    t2 = time.time()
    del r
    del a

    a=db_app.app_db()
    print time.time() - t2, "Creating db (contains", db_app.statdb(a), "Entries)"
    t2 = time.time()
    r=a.root()
    print time.time() - t2, "Creating root"
    t2 = time.time()

    o1 = r.GetObjs(batch=True)
    cc = len(o1)
    for o in o1:
        o2 = o.GetObjs(batch=True)
        cc+=len(o2)
        for a in o2:
            o3 = a.GetObjs(batch=True)
            cc+=len(o3)
    print time.time() - t2, "Loading batch", cc, "objects"
    t2 = time.time()

    user = User(u"test")
    for id in ids:
        r.Delete(id, user=user)
    print time.time() - t2, "Deleting objects recursively"

    print "------------"
    print time.time() - t, "Total. Finished (contains", db_app.statdb(a), "Entries)"
    a.Close()
    print "--------------------------------------------------------"



def test4():
    print "4) Testing objects create / load cache / delete, 3 level, no files"
    t = time.time()
    a=db_app.app_db()
    print time.time() - t, "Creating db (contains", db_app.statdb(a), "Entries)"
    t2 = time.time()
    r=a.root()
    print time.time() - t2, "Creating root"
    t2 = time.time()

    c=r
    ids=[]
    n=0
    c.useCache = True
    for i in range(0,level1):
        o=db_app.createObj1(r)
        o.useCache = True
        ids.append(o.id)
        #print o.id
        n+=1
        for i2 in range(0,level2):
            o2=db_app.createObj1(o)
            o2.useCache = True
            #print o2.id,
            n+=1
            for i3 in range(0,level3_1):
                o3 = db_app.createObj1(o2)
                o3.useCache = True
                #print o3.id,
                n+=1
            for i3 in range(0,level3_2):
                o3 = db_app.createObj2(o2)
                o3.useCache = True
                #print o3.id,
                n+=1

    print time.time() - t2, "Creating", n, "objects"
    t2 = time.time()

    o1 = r.GetObjs()
    cc = len(o1)
    for o in o1:
        o2 = o.GetObjs()
        cc+=len(o2)
        for a in o2:
            o3 = a.GetObjs()
            cc+=len(o3)
    print time.time() - t2, "Loading", cc, "objects"
    t2 = time.time()

    r.Close()
    print time.time() - t2, "Closing", cc, "objects"
    t2 = time.time()

    user = User(u"test")
    for id in ids:
        try:
            r.Delete(id, user=user)
        except Exception, e:
            raise Exception, str(ids)+ " " + str(id)
    print time.time() - t2, "Deleting objects recursively"

    print "------------"
    print time.time() - t, "Total. Finished (contains", db_app.statdb(a), "Entries)"
    a.Close()
    print "--------------------------------------------------------"


def test5():
    print "5) Testing objects create / close load cache / delete, 3 level, no files"
    t = time.time()
    a=db_app.app_db()
    print time.time() - t, "Creating db (contains", db_app.statdb(a), "Entries)"
    t2 = time.time()
    r=a.root()
    print time.time() - t2, "Creating root"
    t2 = time.time()

    c=r
    ids=[]
    n=0
    c.useCache = True
    for i in range(0,level1):
        o=db_app.createObj1(r)
        o.useCache = True
        ids.append(o.id)
        #print o.id
        n+=1
        for i2 in range(0,level2):
            o2=db_app.createObj1(o)
            o2.useCache = True
            #print o2.id,
            n+=1
            for i3 in range(0,level3_1):
                o3 = db_app.createObj1(o2)
                o3.useCache = True
                #print o3.id,
                n+=1
            for i3 in range(0,level3_2):
                o3 = db_app.createObj2(o2)
                o3.useCache = True
                #print o3.id,
                n+=1

    print time.time() - t2, "Creating", n, "objects"
    t2 = time.time()

    r.Close()
    print time.time() - t2, "Closing", n, "objects"
    t2 = time.time()

    r=a.root()
    r.useCache = True
    o1 = r.GetObjs()
    cc = len(o1)
    for o in o1:
        o.useCache = True
        o2 = o.GetObjs()
        cc+=len(o2)
        for a in o2:
            a.useCache = True
            o3 = a.GetObjs()
            cc+=len(o3)
    print time.time() - t2, "Loading after Close", cc, "objects"
    t2 = time.time()

    user = User(u"test")
    for id in ids:
        r.Delete(id, user=user)
        #try:
        #    r.Delete(id)
        #except Exception, e:
        #    raise Exception, str(ids)+ " " + str(id)
    print time.time() - t2, "Deleting objects recursively"

    print "------------"
    print time.time() - t, "Total. Finished (contains", db_app.statdb(a), "Entries)"
    a.Close()
    print "--------------------------------------------------------"



if __name__ == '__main__':
    db_app.emptypool(db_app.app_db())
    test1()
    test2()
    test3()
    test4()
    test5()
    db_app.emptypool(db_app.app_db())



# -*- coding: utf-8 -*-

import time
import unittest

from nive.cms.tests.db_app import *

from nive.cms.cmsview.view import *
from nive.cms.workflow.view import WorkflowEdit
from nive.cms.workflow import wf

from pyramid import testing 
from pyramid.httpexceptions import HTTPFound
from pyramid.renderers import render



class tWf(unittest.TestCase):

    def setUp(self):
        request = testing.DummyRequest()
        request._LOCALE_ = "en"
        self.config = testing.setUp(request=request)
        self.request = request
        self.app = app(["nive.cms.workflow.wf.wfProcess", "nive.cms.workflow.view"])
        self.app.Startup(self.config)
        self.root = self.app.root()
        user = User(u"test")
        user.groups.append("group:editor")
        self.page = create_page(self.root, user)
        self.request.context = self.page

    def tearDown(self):
        user = User(u"test")
        testing.tearDown()
        root = self.app.root()
        self.root.Delete(self.page.id, user=user)
        self.app.Close()


    def test_functions(self):
        user = User(u"test")
        user.groups.append("group:editor")
        wf.publish(None, self.page, user, {})
        self.assert_(self.page.meta.pool_state==1)
        wf.revoke(None, self.page, user, {})
        self.assert_(self.page.meta.pool_state==0)

    
    def test_views1(self):
        view = WorkflowEdit(self.page, self.request)
        self.assertRaises(HTTPFound, view.action)
        self.assertRaises(HTTPFound, view.publishRecursive)
        view.widget()
        view.workflow()
        
    
    def test_templates(self):
        user = User(u"test")
        user.groups.append("group:editor")
        view = WorkflowEdit(self.page, self.request)
        vrender = {"context":self.page, "view":view, "request": self.request, "cmsview":view}
        values = view.widget()
        values.update(vrender)
        render("nive.cms.workflow:templates/widget.pt", values)
        values = view.workflow()
        values.update(vrender)
        render("nive.cms.workflow:templates/editorpage.pt", values)

        
        

if __name__ == '__main__':
    unittest.main()



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------
__doc__ = """
"""

from nive.i18n import _
from nive.definitions import IPage
from nive.definitions import IToolboxWidgetConf, IEditorWidgetConf
from nive.definitions import ViewModuleConf, ViewConf, WidgetConf
from nive.cms.cmsview.view import Editor


configuration = ViewModuleConf(
    id = "wfview",
    name = _(u"CMS workflow extension"),
    static = "nive.cms.workflow:static",
    permission = "read",
    context = IPage,
    containment = "nive.cms.cmsview.cmsroot.cmsroot",
    view = "nive.cms.workflow.view.WorkflowEdit"
)

configuration.views = [
    ViewConf(name = "wfWidget", attr = "widget",   renderer = "nive.cms.workflow:templates/widget.pt"),
    ViewConf(name = "workflow", attr = "workflow", renderer = "nive.cms.workflow:templates/editorpage.pt"),
    ViewConf(name = "action",   attr = "action"),
    ViewConf(name = "pubr",     attr = "publishRecursive", permission = "edit")
]

configuration.widgets = [
    WidgetConf(name=_("Workflow"), widgetType=IToolboxWidgetConf, apply=(IPage,), viewmapper="wfWidget", id="cms.wf",    sort=250),
    WidgetConf(name=_("Workflow"), widgetType=IEditorWidgetConf,  apply=(IPage,), viewmapper="workflow", id="editor.wf", sort=250)
]


class WorkflowEdit(Editor):

    def widget(self):
        wf = self.context.GetWfInfo(self.User())
        return {u"wf": wf}   
    
    def workflow(self):
        wf = self.context.GetWfInfo(self.User())
        return {u"wf": wf,u"content": u"", u"result": True, u"cmsview":self, u"head": u""}

    def action(self):
        transition = self.GetFormValue(u"t")
        url = self.GetFormValue("redirect_url")
        if not url:
            url = self.PageUrl()
            
        user = self.User()
        self.context.WfAction("", user, transition=transition)
        self.context.CommitInternal(user)
        msg = _(u"OK")
        self.Redirect(url, messages=[msg])
        
    def publishRecursive(self):
        msgs = []
        ok = 0
        user = self.User()
        
        def recursive(self, page, ok):
            for p in page.GetPages(includeMenu=1):
                if not p.meta.pool_state:
                    try:
                        p.WfAction("publish", user)
                        p.CommitInternal(user)
                        msgs.append("Published: " + p.meta.title)
                        ok += 1
                    except Exception, e:
                        msgs.append(u"Failure: "+str(e))
                        pass
                ok = recursive(self, p, ok)
            return ok
        
        ok = recursive(self, self.context, ok)
             
        url = self.GetFormValue("redirect_url")
        if not url:
            url = self.PageUrl()
        msgs.insert(0, "%d pages published!"%ok)
        self.Redirect(url, messages=msgs)
        



#----------------------------------------------------------------------
# Copyright 2012, 2013 Arndt Droullier, Nive GmbH. All rights reserved.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.
#----------------------------------------------------------------------

__doc__ = ""

import types


from pyramid.i18n import get_localizer
from pyramid.threadlocal import get_current_request

from nive.tools import Tool
from nive.helper import FakeLocalizer
from nive.definitions import ToolConf, IApplication
from nive.i18n import _

from nive.utils.utils import FormatBytesForDisplay

configuration = ToolConf()
configuration.id = "cmsstatistics"
configuration.context = "nive.components.tools.cmsstatistics.cmsstatistics"
configuration.name = _(u"CMS Statistics")
configuration.description = _("This function provides a short summary of elements and data contained in the website.")
configuration.apply = (IApplication,)
configuration.data = [
]
configuration.mimetype = "text/html"



class cmsstatistics(Tool):
    """
    """

    def _Run(self, **values):

        try:
            localizer = get_localizer(get_current_request())
        except:
            localizer = FakeLocalizer()

        app = self.app
        datapool = app.db
        conn = datapool.connection
        c = conn.cursor()

        self.stream.write(u"<table>\n")

        sql = "select count(*) from pool_meta"
        c.execute(sql)
        rec = c.fetchall()
        self.stream.write(localizer.translate(_(u"<tr><th>Elements in total</th><td>${value}</td></tr>\n", mapping={u"value": rec[0][0]})))

        sql = "select count(*) from pool_files"
        c.execute(sql)
        rec = c.fetchall()
        self.stream.write(localizer.translate(_(u"<tr><th>Physical files</th><td>${value}</td></tr>\n", mapping={u"value": rec[0][0]})))

        sql = "select sum(size) from pool_files"
        c.execute(sql)
        rec = c.fetchall()
        self.stream.write(localizer.translate(_(u"<tr><th>Physical files size</th><td>${value}</td></tr>\n", mapping={u"value": FormatBytesForDisplay(rec[0][0])})))

        for t in app.GetAllObjectConfs():
            sql = "select count(*) from pool_meta where pool_type='%s'" % t.id
            c.execute(sql)
            rec = c.fetchall()
            self.stream.write(localizer.translate(_(u"<tr><th>${name}</th><td>${value}</td></tr>\n", mapping={u"name": t.name, u"value": rec[0][0]})))
        
        self.stream.write(u"</table>\n")

        c.close()
        return 1

